//
//Tossed borcconade
class BrocconadeFlying extends Projectile config(FoodFight);

#exec AUDIO IMPORT FILE="Sounds\ticktockA.WAV" NAME=ticktockA
#exec AUDIO IMPORT FILE="Sounds\alarmclock.WAV" NAME=alarmclock

var bool bCanHitOwner, bHitWater, Detonating;
var float Count, SmokeRate;

var() bool CanHitInstigator;
var() float HitInstigatorTimeOut;

var config int BrocBalls;

//Shares that it is exploding in case it's done early from collision, etc
Replication
{
   UnReliable if (Role == ROLE_Authority)
      Detonating;
}

simulated function PostBeginPlay()
{
	local vector X,Y,Z;
	local rotator RandRot;

	Super.PostBeginPlay();
	if (Level.NetMode != NM_DedicatedServer)
		PlayAnim('Still');
	SetTimer(1.0,true);
	Detonating= False;

	if (Role == ROLE_Authority)
	{
		GetAxes(Instigator.ViewRotation,X,Y,Z);
		Velocity = Vector(Rotation) * Speed;
		Velocity.z += 200;
		RandRot.Pitch = FRand() * 1400 - 700;
      RandRot.Yaw = FRand() * 1400 - 700;
      RandRot.Roll = FRand() * 1400 - 700;
		MaxSpeed = 1000;
		Velocity = Velocity >> RandRot;
		RandSpin(50000);
		bCanHitOwner = False;
		if (Instigator.HeadRegion.Zone.bWaterZone)
		{
			bHitWater = True;
			Disable('Tick');
			Velocity=0.6*Velocity;
		}
	}
}

simulated function ZoneChange(Zoneinfo NewZone)
{
	local waterring w;
	
	if (!NewZone.bWaterZone || bHitWater) Return;
	bHitWater = True;
	w = Spawn(class'WaterRing',,,,rot(16384,0,0));
	w.DrawScale = 0.2;
	w.RemoteRole = ROLE_None;
	Velocity=0.1*Velocity;
}

//Spawns effects on the flying broc
simulated function Tick(float DeltaTime)
{
	local BlackSmoke b;
	local SmallSpark2 c;
	local FlyingLeaf d;

	if ( bHitWater || Level.bDropDetail ) 
	{
		Disable('Tick');
		Return;
	}
	//happens often
	if (FRand()<0.05)
	{
		b = Spawn(class'BlackSmoke');
		b.RemoteRole = ROLE_None;
		c = Spawn(class'SmallSpark2');
		c.RemoteRole = ROLE_None;
	}
	//doesn't happen as much
	if (FRand()<0.02)
	{
		d = Spawn(class'FlyingLeaf');
		d.RemoteRole = ROLE_None;
	}
}

function Timer()
{
	//plays an audible alarm that it's about to explode and changes the skin to flashy red
	if (LifeSpan <= 4)
	{
		Disable('Tick');
		AmbientSound=Sound'alarmclock';
		SetDisplayProperties(ERenderStyle.STY_Translucent,FireTexture'UnrealShare.Belt_fx.ShieldBelt.RedShield',true,true);
	}

	//blows up. I called it early (at 2) because I wanted to be sure it exploded and didn't delete due to lifespan ending
	if (LifeSpan <= 2) //She's gonna blow
	{
		Detonating = True;
		Explosion(Location+Vect(0,0,1)*16);
	}
}

simulated function Landed(vector HitNormal)
{
	HitWall(HitNormal, None);
}

simulated function ProcessTouch(actor Other, vector HitLocation)
{
	if ((!CanHitInstigator || (CanHitInstigator && (Default.LifeSpan - LifeSpan) < HitInstigatorTimeOut)) && Other == Instigator)
		return;

	if ((Other!=instigator) || bCanHitOwner)
		Explosion(HitLocation);
}

//wall bounce, it's pretty active and rebounds a good bit
simulated function HitWall(vector HitNormal, actor Wall)
{
	bCanHitOwner = True;
	Velocity = 0.7*(( Velocity dot HitNormal ) * HitNormal * (-2.0) + Velocity);   // Reflect off Wall w/damping
	RandSpin(100000);
	speed = VSize(Velocity);
	if (Level.NetMode != NM_DedicatedServer)
		PlaySound(ImpactSound, SLOT_Misc, 1.5 );
	if (Velocity.Z > 400)
		Velocity.Z = 0.5 * (400 + Velocity.Z);	
	else if (speed < 20)
	{
		bBounce = False;
		SetPhysics(PHYS_None);
	}
}

//Explosion effects
function Explosion(vector HitLocation)
{
	local int i;
	local vector Start;

	Spawn(class'TOMShockWaveBrocconade',,,HitLocation);
	Spawn(class'UT_SpriteSmokePuff',,,HitLocation);
	//spawns number of balls declared by the ini file
	for (i=0; i<(BrocBalls); i++)
	{
		Start = HitLocation + 4 * VRand();
    	Spawn(class'BrocconadeShard',,,Start,Rotator(Start - Location));
	}
	Destroy();
}

defaultproperties
{
	BrocBalls=7
	HitInstigatorTimeOut=0.500000
	CanHitInstigator=True
   speed=1000.000000
   MaxSpeed=1000.000000
	Damage=100.00
	MomentumTransfer=50000
	LifeSpan=8.00000
   MyDamageType=BrocconadeDeath
   ImpactSound=Sound'ThumpC'
   ExplosionDecal=Class'FFBlastMark'
	Physics=PHYS_Falling
	RemoteRole=ROLE_SimulatedProxy
	AnimSequence=Still
	Mesh=LodMesh'Brocconade'
   AmbientGlow=64
   bUnlit=True
   bBounce=True
	CollisionRadius=8.00
	CollisionHeight=19.00
   bNetTemporary=False
   bFixedRotationDir=True
   DesiredRotation=(Pitch=12000,Yaw=5666,Roll=2334) //check me
   AmbientSound=Sound'ticktockA'
   SoundPitch=64
   SoundRadius=120
   SoundVolume=150
}
