//
//Less bouncy flying broc
class BrocconadeFlyingAlt extends Projectile config(FoodFight);

var bool bCanHitOwner, bHitWater, Detonating;
var float Count, SmokeRate;

var() bool CanHitInstigator;
var() float HitInstigatorTimeOut;
var config int BrocBallsAlt;

Replication
{
   UnReliable if (Role == ROLE_Authority)
      Detonating;
}

simulated function PostBeginPlay()
{
	local vector X,Y,Z;
	local rotator RandRot;

	Super.PostBeginPlay();
	if (Level.NetMode != NM_DedicatedServer)
		PlayAnim('Still');
	SetTimer(1.0,true);
	Detonating= False;

	if (Role == ROLE_Authority)
	{
		GetAxes(Instigator.ViewRotation,X,Y,Z);	
		Velocity = X * (Instigator.Velocity Dot X)*0.3 + Vector(Rotation) * (Speed + FRand() * 100);
		Velocity.z += 210;  //Tossforce
		RandRot.Pitch = FRand() * 1400 - 700;
      RandRot.Yaw = FRand() * 1400 - 700;
      RandRot.Roll = FRand() * 1400 - 700;
		MaxSpeed = 1000;
		Velocity = Velocity >> RandRot;
		RandSpin(50000);
		bCanHitOwner = False;
		if (Instigator.HeadRegion.Zone.bWaterZone)
		{
			bHitWater = True;
			Disable('Tick');
			Velocity=0.6*Velocity;			
		}
	}
}

simulated function ZoneChange(Zoneinfo NewZone)
{
	local waterring w;
	
	if (!NewZone.bWaterZone || bHitWater) Return;
	bHitWater = True;
	w = Spawn(class'WaterRing',,,,rot(16384,0,0));
	w.DrawScale = 0.2;
	w.RemoteRole = ROLE_None;
	Velocity=0.1*Velocity;
}

simulated function Tick(float DeltaTime)
{
	local BlackSmoke b;
	local SmallSpark2 c;

	if ( bHitWater || Level.bDropDetail )
	{
		Disable('Tick');
		Return;
	}
	if (FRand()<0.05)
	{
		b = Spawn(class'BlackSmoke');
		b.RemoteRole = ROLE_None;
		c = Spawn(class'SmallSpark2');
		c.RemoteRole = ROLE_None;
	}
}

function Timer()
{
	if (LifeSpan <= 4) //alarm sounding, effects stop
	{
		Disable('Tick');
		AmbientSound=Sound'alarmclock';
		SetDisplayProperties(ERenderStyle.STY_Translucent,FireTexture'UnrealShare.Belt_fx.ShieldBelt.RedShield',true,true);
	}


	if (LifeSpan <= 2) //She's gonna blow
	{
		Detonating = True;
		Explosion(Location+Vect(0,0,1)*16);
	}
}

simulated function Landed(vector HitNormal)
{
	HitWall(HitNormal, None);
}

simulated function ProcessTouch(actor Other, vector HitLocation)
{
	if ((!CanHitInstigator || (CanHitInstigator && (Default.LifeSpan - LifeSpan) < HitInstigatorTimeOut)) && Other == Instigator)
		return;

	if ((Other!=instigator) || bCanHitOwner)
		Explosion(HitLocation);
}

simulated function HitWall(vector HitNormal, actor Wall)
{
	bCanHitOwner = True;
	Velocity = 0.2*(( Velocity dot HitNormal ) * HitNormal * (-2.0) + Velocity);   // Reflect off Wall w/damping
	RandSpin(100000);
	speed = VSize(Velocity);
	if (Level.NetMode != NM_DedicatedServer)
		PlaySound(ImpactSound, SLOT_Misc, 1.5 );
	if (Velocity.Z > 400)
		Velocity.Z = 0.5 * (400 + Velocity.Z);	
	else if (speed < 20)
	{
		bBounce = False;
		SetPhysics(PHYS_None);
	}
}

function Explosion(vector HitLocation)
{
	local int i;
	local vector Start;

	Spawn(class'TOMShockWaveBrocconade',,,HitLocation);
	Spawn(class'UT_SpriteSmokePuff',,,HitLocation);
	for (i=0; i<(BrocBallsAlt); i++)
	{
		Start = HitLocation + 4 * VRand();
    	Spawn(class'BrocconadeShard',,,Start,Rotator(Start - Location));
	}
	Destroy();
}

defaultproperties
{
	BrocBallsAlt=7
	HitInstigatorTimeOut=0.500000
	CanHitInstigator=True
   speed=600.000000
   MaxSpeed=1000.000000
	Damage=100.00
	MomentumTransfer=50000
	LifeSpan=8.00000
   MyDamageType=BrocconadeDeath
   ImpactSound=Sound'ThumpC'
   ExplosionDecal=Class'FFBlastMark'
	Physics=PHYS_Falling
	RemoteRole=ROLE_SimulatedProxy
	AnimSequence=Still
	Mesh=LodMesh'Brocconade'
   AmbientGlow=64
   bUnlit=True
   bBounce=True
	CollisionRadius=8.00
	CollisionHeight=19.00
   bNetTemporary=False
   bFixedRotationDir=True
   DesiredRotation=(Pitch=12000,Yaw=5666,Roll=2334)
   AmbientSound=Sound'ticktockA'
   SoundPitch=64
   SoundRadius=120
   SoundVolume=150
}
