//
//main class for the mod, but much of the code happens in FlagMutatorCTFGame too
class BurgerCTFGame extends FlagMutatorCTFGame config;

#exec AUDIO IMPORT FILE="Sounds\BurgerReset.wav" 	NAME="BurgerReset" 	GROUP="Announcer"
#exec AUDIO IMPORT FILE="Sounds\blueroll.wav" 		NAME="blueroll" 		GROUP="Announcer"
#exec AUDIO IMPORT FILE="Sounds\redroll.wav" 		NAME="redroll" 		GROUP="Announcer"
#exec AUDIO IMPORT FILE="Sounds\blueround.wav" 		NAME="blueround" 		GROUP="Announcer"
#exec AUDIO IMPORT FILE="Sounds\redround.wav" 		NAME="redround" 		GROUP="Announcer"
#exec AUDIO IMPORT FILE="Sounds\bluescores.wav" 	NAME="bluescores" 	GROUP="Announcer"
#exec AUDIO IMPORT FILE="Sounds\redscores.wav" 		NAME="redscores" 		GROUP="Announcer"
#exec AUDIO IMPORT FILE="Sounds\redwinner.wav" 		NAME="redwinner" 		GROUP="Announcer"
#exec AUDIO IMPORT FILE="Sounds\bluewinner.wav" 	NAME="bluewinner" 	GROUP="Announcer"

function InitGameReplicationInfo()
{
	Super.InitGameReplicationInfo();
	BurgerReplicationInfo(GameReplicationInfo).RealGameName = Default.GameName;
}

//runs the setup for the chosen map, setting the burger spawn
function PreBeginPlay()
{
	Super.PreBeginPlay();
	Spawn(class'FoodFight.BurgerCTFMapSetup');
}

function PlayStartUpMessage(PlayerPawn NewPlayer)
{
	local int i;
	local color WhiteColor;

	NewPlayer.ClearProgressMessages();

	// GameName
	NewPlayer.SetProgressMessage(Default.GameName, i++);
	if (bRequireReady && (Level.NetMode != NM_Standalone))
		NewPlayer.SetProgressMessage(TourneyMessage, i++);
	else
		NewPlayer.SetProgressMessage(StartUpMessage, i++);

	if (GoalTeamScore > 0)
		NewPlayer.SetProgressMessage(int(GoalTeamScore)@GameGoal, i++);

	if (NewPlayer.PlayerReplicationInfo.Team < 4)
	{
		NewPlayer.SetProgressColor(class'ChallengeTeamHUD'.Default.TeamColor[NewPlayer.PlayerReplicationInfo.Team], i);
		NewPlayer.SetProgressMessage(StartupTeamMessage@Teams[NewPlayer.PlayerReplicationInfo.Team].TeamName$StartupTeamTralier, i++);
		WhiteColor.R = 255;
		WhiteColor.G = 255;
		WhiteColor.B = 255;
		NewPlayer.SetProgressColor(WhiteColor, i);
		if (!bRatedGame)
			NewPlayer.SetProgressMessage(TeamChangeMessage, i++);
	}

	if (Level.NetMode == NM_Standalone)
		NewPlayer.SetProgressMessage(SingleWaitingMessage, i++);
}

//custom scoring and announcements
function ScoreFlag(Pawn Scorer, CTFFlag theFlag)
{
	local pawn TeamMate;
	local Actor A;
	local BaseFlag CapturePlace;
	local int i;
	local Pawn P;

	if ( bRatedGame && Scorer.IsA('PlayerPawn') )
		bFulfilledSpecial = true;
	Scorer.PlayerReplicationInfo.Score += 7;

	Teams[Scorer.PlayerReplicationInfo.Team].Score += 1.0;

	//Custom round winner announcements
	if (Teams[Scorer.PlayerReplicationInfo.Team].Score < GoalTeamScore)
	{
		if (Scorer.PlayerReplicationInfo.Team == 0) //Red team scored
		{
			if (FRand() > 0.66)
			{
				for(P=Level.PawnList; P!=none; P=P.nextPawn)
					if (P.bIsPlayer && P.IsA('PlayerPawn'))
						PlayerPawn(P).ClientPlaySound(Sound'redscores');
			}
			else if (FRand() > 0.33)
			{
				for(P=Level.PawnList; P!=none; P=P.nextPawn)
					if (P.bIsPlayer && P.IsA('PlayerPawn'))
						PlayerPawn(P).ClientPlaySound(Sound'redroll');
			}
			else
			{
				for(P=Level.PawnList; P!=none; P=P.nextPawn)
					if (P.bIsPlayer && P.IsA('PlayerPawn'))
						PlayerPawn(P).ClientPlaySound(Sound'redround');
			}
		}
		else if (Scorer.PlayerReplicationInfo.Team == 1) //Blue team scored
		{
			if (FRand() > 0.66)
			{
				for(P=Level.PawnList; P!=none; P=P.nextPawn)
					if (P.bIsPlayer && P.IsA('PlayerPawn'))
						PlayerPawn(P).ClientPlaySound(Sound'bluescores');
			}
			else if (FRand() > 0.33)
			{
				for(P=Level.PawnList; P!=none; P=P.nextPawn)
					if (P.bIsPlayer && P.IsA('PlayerPawn'))
						PlayerPawn(P).ClientPlaySound(Sound'blueroll');
			}
			else
			{
				for(P=Level.PawnList; P!=none; P=P.nextPawn)
					if (P.bIsPlayer && P.IsA('PlayerPawn'))
						PlayerPawn(P).ClientPlaySound(Sound'blueround');
			}
		}
	}
	else if (Teams[Scorer.PlayerReplicationInfo.Team].Score >= GoalTeamScore) //replaces end game sounds
	{
		if (Scorer.PlayerReplicationInfo.Team == 0) //Red team scored
		{
			for(P=Level.PawnList; P!=none; P=P.nextPawn)
				if (P.bIsPlayer && P.IsA('PlayerPawn'))
					PlayerPawn(P).ClientPlaySound(Sound'redwinner');
		}
		else if (Scorer.PlayerReplicationInfo.Team == 1) //Blue team scored
		{
			for(P=Level.PawnList; P!=none; P=P.nextPawn)
				if (P.bIsPlayer && P.IsA('PlayerPawn'))
					PlayerPawn(P).ClientPlaySound(Sound'bluewinner');
		}
	}

	if (NeutralFlag(theFlag) != None)
	{
		NeutralFlag(theFlag).ClearAssistance(Scorer.PlayerReplicationInfo.Team);
		for (i = 0; i < ArrayCount(NeutralFlag(theFlag).AssistedBy); i++) 
		{
			TeamMate = NeutralFlag(theFlag).AssistedBy[i];
			if (TeamMate == None)
				break;
			if (TeamMate != Scorer)
			{
				TeamMate.PlayerReplicationInfo.Score += 3;	// score points for assist
			}
		}
	}
	
	foreach AllActors(class'BaseFlag', CapturePlace)
		for (i = 0; i < 4; i++)
			if (CapturePlace.Touching[i] == Scorer)
				break;
	
	if (CapturePlace == None)
		A = Scorer;
	else
		A = CapturePlace;
	
	MutatorFlagCapture(theFlag, Scorer, A);
	MutatorScoreCapture(Scorer, 1, A);
	
	for (TeamMate = Level.PawnList; TeamMate != None; TeamMate = TeamMate.NextPawn) 
	{
		if (TeamMate.IsA('PlayerPawn'))
		{
			PlayerPawn(TeamMate).ClientPlaySound(CaptureSound[Scorer.PlayerReplicationInfo.Team]);
		}
		else if (TeamMate.IsA('Bot'))
			Bot(TeamMate).SetOrders(BotReplicationInfo(TeamMate.PlayerReplicationInfo).RealOrders, BotReplicationInfo(TeamMate.PlayerReplicationInfo).RealOrderGiver, true);
	}

	if (Level.Game.WorldLog != None)
		Level.Game.WorldLog.LogSpecialEvent("flag_captured", Scorer.PlayerReplicationInfo.PlayerID, Teams[Scorer.PlayerReplicationInfo.Team].TeamIndex);
	if (Level.Game.LocalLog != None)
		Level.Game.LocalLog.LogSpecialEvent("flag_captured", Scorer.PlayerReplicationInfo.PlayerID, Teams[Scorer.PlayerReplicationInfo.Team].TeamIndex);
	
	EndStatsClass.Default.TotalFlags++;
	
	BroadcastLocalizedMessage(class'BurgerMessage', 0, Scorer.PlayerReplicationInfo, None, Teams[Scorer.PlayerReplicationInfo.Team]);
	if (theFlag.HomeBase.Event != '')
		foreach allactors(class'Engine.Actor', A, theFlag.HomeBase.Event )
			A.Trigger(theFlag.HomeBase,	Scorer);

	if ( (bOverTime || GoalTeamScore != 0) && Teams[Scorer.PlayerReplicationInfo.Team].Score >= GoalTeamScore )
		EndGame("teamscorelimit");
	else if ( bOverTime )
		EndGame("timelimit");
}

function bool SetEndCams(string Reason)
{
	local TeamInfo Best;
	local CTFFlag BestFlag;
	local actor CamActor;
	local Pawn P;
	local int i;
	local byte bFixedCamera;
	local PlayerPawn Player;

	for (i = 0; i < MaxTeams; i++)
		if ( Best == None || Best.Score < Teams[i].Score )
			Best = Teams[i];

	for (i = 0; i < MaxTeams; i++)
		if ( Best.TeamIndex != i && Best.Score == Teams[i].Score ) 
		{
			BroadcastLocalizedMessage(DMMessageClass, 0);
			return false;
		}		

	// find winner
	ForEach AllActors(class'CTFFlag', BestFlag)
		if ( BestFlag.Team == Best.TeamIndex )
			break;

	GameReplicationInfo.GameEndedComments = TeamPrefix@Best.TeamName@GameEndedMessage;
	EndTime = Level.TimeSeconds + 3.0;
	
	CamActor = SetupEndCam(BestFlag, BurgerReplicationInfo(GameReplicationInfo).NeutralFlag, bFixedCamera);
	
	for (P = Level.PawnList; P != None; P = P.nextPawn) 
	{
		P.GotoState('GameEnded');
		Player = PlayerPawn(P);
		if ( Player != None ) 
		{
			Player.ViewTarget = CamActor;
			Player.bFixedCamera = (bFixedCamera != 0);
			Player.bBehindView = (bFixedCamera == 0);
			Player.ClientGameEnded();
		}
	}
	CalcEndStats();
	return true;
}

function Actor SetupEndCam(CTFFlag BestFlag, CTFFlag NeutralFlag, out byte bFixed)
{
	local Actor ViewFrom, ViewDest;
	local BurgerEndCam theCam;
	local Pawn P;
	
	if ( NeutralFlag.Holder != None ) 
	{
		ViewFrom = NeutralFlag.Holder;
		if ( FastTrace(ViewFrom.Location, BestFlag.HomeBase.Location) ) 
		{
			if ( VSize(ViewFrom.Location - BestFlag.HomeBase.Location) > 2000 )
				ViewDest = BestFlag.HomeBase;
			else 
			{
				ViewDest = ViewFrom;
				ViewFrom = BestFlag.HomeBase;
			}
		}
		else if ( VSize(ViewFrom.Location - NeutralFlag.HomeBase.Location) < 2000
				&& FastTrace(ViewFrom.Location, NeutralFlag.HomeBase.Location) ) 
		{
			ViewDest = ViewFrom;
			ViewFrom = NeutralFlag.HomeBase;
		}
		else
			return NeutralFlag.Holder;
		bFixed = 1;
	}
	else 
	{
		ViewFrom = NeutralFlag.HomeBase;
		if ( FastTrace(ViewFrom.Location, BestFlag.HomeBase.Location) )
			ViewDest = BestFlag.HomeBase;
		else 
		{
			for (P = Level.PawnList; P != None; P = P.NextPawn)
				if ( P.Health > 0 && !P.bHidden && P.bIsPlayer && P.PlayerReplicationInfo != None
						&& P.PlayerReplicationInfo.Team == BestFlag.Team
						&& FastTrace(ViewFrom.Location, P.Location) ) 
				{
					ViewDest = P;
					break;
				}
			if ( ViewDest != None )
				bFixed = 1;
		}
	}
	if ( bFixed == 0 )
		return ViewFrom;
	
	theCam = Spawn(class'BurgerEndCam');
	if ( theCam != None && theCam.SetupCam(ViewFrom.Location, ViewDest.Location) )
		return theCam;
	
	return ViewFrom;
}

//lots of code to help the bots play properly
function bool FindSpecialAttractionFor(Bot aBot)
{
	local CTFFlag FriendlyFlag, EnemyFlag, NeutralFlag;
	local bool bSeeFlag, bReachHome, bOrdered, bLogThis;
	local float Dist, HomeSize;

	if ( aBot.LastAttractCheck == Level.TimeSeconds )
		return false;
	aBot.LastAttractCheck = Level.TimeSeconds;

	FriendlyFlag = CTFReplicationInfo(GameReplicationInfo).FlagList[aBot.PlayerReplicationInfo.Team];
	NeutralFlag = BurgerReplicationInfo(GameReplicationInfo).NeutralFlag;

	if ( aBot.PlayerReplicationInfo.Team == 0 )
		EnemyFlag = CTFReplicationInfo(GameReplicationInfo).FlagList[1];
	else
		EnemyFlag = CTFReplicationInfo(GameReplicationInfo).FlagList[0];
	
	HomeSize = VSize(FriendlyFlag.HomeBase.Location - NeutralFlag.HomeBase.Location) * 0.5;

	bOrdered = aBot.bSniping || aBot.Orders == 'Follow' || aBot.Orders == 'Hold';
	
	// the enemy has the flag, try to stop him
	if ( !NeutralFlag.bHome && NeutralFlag.Holder != None
			&& NeutralFlag.Holder.PlayerReplicationInfo.Team != aBot.PlayerReplicationInfo.Team )
	{
		bSeeFlag = aBot.LineOfSightTo(NeutralFlag.Position());
		NeutralFlag.bKnownLocation = NeutralFlag.bKnownLocation || bSeeFlag;
		
		if ( bSeeFlag ) 
		{
			NeutralFlag.bKnownLocation = true;
			if ( Level.TimeSeconds - LastSeeFlagCarrier > 6 ) 
			{
				LastSeeFlagCarrier = Level.TimeSeconds;
				if ( (NeutralFlag.HomeBase.Region.Zone != FriendlyFlag.HomeBase.Region.Zone
						&& NeutralFlag.Holder.Region.Zone == FriendlyFlag.HomeBase.Region.Zone)
						|| VSize(NeutralFlag.Holder.Location - FriendlyFlag.HomeBase.Location)
							< VSize(NeutralFlag.HomeBase.Location - FriendlyFlag.HomeBase.Location) * 0.5 )
					aBot.SendTeamMessage(None, 'OTHER', 14, 10);	// "Incoming!"
				else
					aBot.SendTeamMessage(None, 'OTHER', 12, 10);	// "Enemy flag carrier is here."
			}
			aBot.SetEnemy(NeutralFlag.Holder);
			aBot.Orders = 'Freelance';
			aBot.MoveTarget = NeutralFlag.Holder;
			if ( aBot.IsInState('Attacking') )
				return false;
			else 
			{
				aBot.GotoState('Attacking');
				return true;
			}
		}
		else if ( aBot.Orders == 'Defend' && (NeutralFlag.bKnownLocation || FRand() < 0.1) ) 
		{
			NeutralFlag.bKnownLocation = true;
			aBot.MoveTarget = aBot.FindPathToward(NeutralFlag.Position());
			if ( Level.TimeSeconds - LastAlertEmptyBase > 6
					&& GetPlayersAtHome(FriendlyFlag, HomeSize, True) * 2
						< GetPlayersAtHome(FriendlyFlag, HomeSize, False) - 2 ) 
			{
				LastAlertEmptyBase = Level.TimeSeconds;
				if ( !PlayerIsAtHome(FriendlyFlag, aBot, HomeSize) )
					aBot.SendTeamMessage(None, 'OTHER', 1, 10);	// "Get our flag back." - base got overrun by enemies
				else
					aBot.SendTeamMessage(None, 'OTHER', 13, 10);	// "Need backup." - bot needs help defending
			}
			else if ( Level.TimeSeconds - LastAlertEmptyBase > 6
					&& GetPlayersAtHome(FriendlyFlag, HomeSize, True) == 0 ) 
			{
				LastAlertEmptyBase = Level.TimeSeconds;
				aBot.SendTeamMessage(None, 'OTHER', 0, 10);	// "Base is uncovered!" - no team mate in own base
			}
			else if ( Level.TimeSeconds - LastAlertEmptyBase < 6
					&& GetPlayersAtHome(FriendlyFlag, HomeSize, False) == 0
					&& PlayerIsAtHome(FriendlyFlag, aBot, HomeSize) )
				aBot.SendTeamMessage(None, 'OTHER', 8, 0);	// "Got our flag" - base is not uncovered anymore
			if ( aBot.MoveTarget != None ) 
			{
				SetAttractionStateFor(aBot);
				return true;
			}
		}
	}
	
	// Bot has the flag, so he tries to break into the enemy's base to capture it
	if ( NeutralFlag.Holder == aBot ) 
	{
		aBot.bCanTranslocate = false;
		bReachHome = aBot.ActorReachable(EnemyFlag.HomeBase);
		if ( bReachHome && VSize(aBot.Location - EnemyFlag.Location) < 30 )
			EnemyFlag.Touch(aBot);
		if ( aBot.Enemy != None ) 
		{
			if ( aBot.Health < 60 )
				aBot.SendTeamMessage(None, 'OTHER', 13, 25);	// "Need backup." - bot needs help breaking into enemy base
			if ( !aBot.IsInState('FallBack') ) 
			{
				aBot.bNoClearSpecial = true;
				aBot.TweenToRunning(0.1);
				aBot.GotoState('Fallback', 'SpecialNavig');
			}
			if ( bReachHome )
				aBot.MoveTarget = EnemyFlag.HomeBase;
			else
				return FindPathToBase(aBot, EnemyFlag.HomeBase);
		}
		else 
		{
			if ( !aBot.IsInState('Roaming') ) 
			{
				aBot.bNoClearSpecial = true;
				aBot.TweenToRunning(0.1);
				aBot.GotoState('Roaming', 'SpecialNavig');
			}
			if ( bReachHome )
				aBot.MoveTarget = EnemyFlag.HomeBase;
			else
				return FindPathToBase(aBot, EnemyFlag.HomeBase);
		}		
		return true;
	}
	
	// the flag is not carried by a player, so the bot will try to get it, if he was ordered to
	if ( NeutralFlag.Holder == None ) 
	{
		if ( aBot.ActorReachable(NeutralFlag) ) 
		{
			aBot.MoveTarget = NeutralFlag;
			SetAttractionStateFor(aBot);
			return true;
		}
		else if ( aBot.Orders == 'Attack' || (aBot.Orders == 'Follow' && aBot.OrderObject.IsA('Bot')
				&& (Pawn(aBot.OrderObject).Health <= 0 || (NeutralFlag.Region.Zone == aBot.Region.Zone
				&& VSize(NeutralFlag.Location - aBot.Location) < 2000))) ) 
		{
			aBot.MoveTarget = aBot.FindPathToward(NeutralFlag);
			if ( aBot.MoveTarget != None ) 
			{
				SetAttractionStateFor(aBot);
				return true;
			}
			else 
			{
				if ( aBot.bVerbose || bLogThis )
					log(aBot$" no path to flag");
				return false;
			}
		}
		return false;
	}

	if ( (bOrdered && !aBot.OrderObject.IsA('Bot')) || aBot.Weapon == None || aBot.Weapon.AIRating < 0.4 )
		return false;

	if ( aBot.Enemy == None && aBot.Orders != 'Defend' && NeutralFlag.Holder != None
			&& NeutralFlag.Holder.PlayerReplicationInfo.Team == aBot.PlayerReplicationInfo.Team ) 
	{
		Dist = VSize(aBot.Location - NeutralFlag.Holder.Location);
		if ( Dist > 500 || VSize(NeutralFlag.Holder.Velocity) > 230
				|| !aBot.LineOfSightTo(NeutralFlag.Holder) ) 
		{
			aBot.MoveTarget = aBot.FindPathToward(NeutralFlag.Holder);
			if ( !aBot.IsInState('Roaming') ) 
			{
				aBot.bNoClearSpecial = true;
				aBot.TweenToRunning(0.1);
				aBot.GotoState('Roaming', 'SpecialNavig');
				return true;
			}
			return (aBot.MoveTarget != None);
		}
		else 
		{
			if ( !aBot.bInitLifeMessage ) 
			{
				aBot.bInitLifeMessage = true;
				aBot.SendTeamMessage(NeutralFlag.Holder.PlayerReplicationInfo, 'OTHER', 3, 10);	// "I've got your back."
			}
			if ( FRand() < 0.35 )
				aBot.GotoState('Wandering');
			else 
			{
				aBot.CampTime = 1.0;
				aBot.bCampOnlyOnce = true;
				aBot.GotoState('Roaming', 'Camp');
			}
			return true;
		}
	}

	return false;
}		

function byte PriorityObjective(Bot aBot)
{
	local CTFFlag EnemyFlag, NeutralFlag;

	NeutralFlag = BurgerReplicationInfo(GameReplicationInfo).NeutralFlag;
	
	if ( aBot.PlayerReplicationInfo.Team == 0 )
		EnemyFlag = CTFReplicationInfo(GameReplicationInfo).FlagList[1];
	else
		EnemyFlag = CTFReplicationInfo(GameReplicationInfo).FlagList[0];
	
	if ( aBot.PlayerReplicationInfo.HasFlag == NeutralFlag ) 
	{
		if ( VSize(aBot.Location - EnemyFlag.HomeBase.Location) < 2000
				&& aBot.LineOfSightTo(EnemyFlag.HomeBase) )
			return 255;
		return 2;
	}

	if ( NeutralFlag.Holder != None )
		return 1;

	return 0;
}

defaultproperties
{
	StartupMessage="Deliver the burger bomb to the other base"
	TourneyMessage="Waiting for other burger fighters"
	gamegoal="deliveries wins the fight!"
	StartMessage="The fight is on!"
   FlagNotifyClass=Class'BurgerNotify'
   ScoreBoardType=Class'BurgerCTFScoreboard'
   RulesMenuType="FoodFight.BurgerCTFRulesSC"
   HUDType=Class'BurgerCTFHUD'
   GameName="Burger CTF"
   GameReplicationInfoClass=Class'BurgerReplicationInfo'
}
