//
//game hud, adds the peppermill effect and various messaging/hud things
class BurgerCTFHUD extends ChallengeCTFHUD;

#exec TEXTURE IMPORT NAME=I_HomeB 		FILE=Textures\I_HomeB.bmp 			GROUP="Icons" FLAGS=2 MIPS=OFF
#exec TEXTURE IMPORT NAME=I_Cap 		FILE=Textures\I_Cap.bmp 				GROUP="Icons" FLAGS=2 MIPS=OFF
#exec TEXTURE IMPORT NAME=I_DownTeamB 	FILE=Textures\I_DownTeamB.bmp 	GROUP="Icons" FLAGS=2 MIPS=OFF

#exec OBJ LOAD FILE=..\TEXTURES\FoodFightTextures.utx PACKAGE=FoodFight.Addons

function Timer()
{
	Super(ChallengeTeamHUD).Timer();
	
	if ( PlayerOwner == None || PawnOwner == None )
		return;
	if ( PawnOwner.PlayerReplicationInfo.HasFlag != None )
		PlayerOwner.ReceiveLocalizedMessage(class'BurgerMessage2', 0);
	else if ( MyFlag != None && MyFlag.Holder != None ) 
	{
		if ( MyFlag.Holder.PlayerReplicationInfo.Team != PawnOwner.PlayerReplicationInfo.Team )
			PlayerOwner.ReceiveLocalizedMessage(class'BurgerMessage2', 1);
	}
}

//adds the burger icon on the right side and changes it depending on who has it currently
simulated function PostRender(canvas Canvas)
{
	local int X, Y, i;
	local bool bAlt;
	
	Super(ChallengeTeamHUD).PostRender(Canvas);		

	if ( PlayerOwner == None || PawnOwner == None || PlayerOwner.GameReplicationInfo == None
		|| PawnOwner.PlayerReplicationInfo == None
		|| ((PlayerOwner.bShowMenu || PlayerOwner.bShowScores) && Canvas.ClipX < 640) )
		return;

	Canvas.Style = Style;

	if ( !bHideHUD && !bHideTeamInfo )
	{
		X = Canvas.ClipX - 70 * Scale;
		Y = Canvas.ClipY - 425 * Scale;
			
		MyFlag = BurgerReplicationInfo(PlayerOwner.GameReplicationInfo).NeutralFlag;
		if ( MyFlag != None )
		{
			Canvas.SetPos(X, Y);
			Canvas.DrawColor.R = 255;
			Canvas.DrawColor.G = 255;
			Canvas.DrawColor.B = 255;
			//if ((MyFlag.bHome) || (!MyFlag.bHeld))
			if (MyFlag.bHome)
				Canvas.DrawIcon(texture'I_HomeB', Scale * 2);

			if ( !MyFlag.bHome && NeutralFlag(MyFlag) != None && NeutralFlag(MyFlag).LastTeam != 4 )
			{
				Canvas.DrawColor = TeamColor[NeutralFlag(MyFlag).LastTeam];
				Canvas.SetPos(X,Y);

				if ( MyFlag.bHeld )
					Canvas.DrawIcon(texture'I_Cap', Scale * 2);
				else
					Canvas.DrawIcon(texture'I_DownTeamB', Scale * 2);
			}
		}
	}
}

simulated function DrawTeam(Canvas Canvas, TeamInfo TI)
{
	local float XL, YL;

	if ( TI != None && TI.Size > 0 ) 
	{
		Canvas.DrawColor = TeamColor[TI.TeamIndex];
		DrawBigNum(Canvas, int(TI.Score), Canvas.ClipX - 144 * Scale, Canvas.ClipY - 336 * Scale - (150 * Scale * TI.TeamIndex), 1);
	}
}

//updates the doll to use the new pickups
simulated function DrawStatus(Canvas Canvas)
{
   local float StatScale, ChestAmount, ThighAmount, H1, H2, X, Y, DamageTime;
   Local int ArmorAmount,CurAbs,i;
   Local inventory Inv,BestArmor;
   local bool bChestArmor, bShieldbelt, bThighArmor, bJumpBoots, bHasDoll;
   local Bot BotOwner;
   local TournamentPlayer TPOwner;
   local texture Doll, DollBelt;

   ArmorAmount = 0;
   CurAbs = 0;
   i = 0;
   BestArmor=None;
   for(Inv=PawnOwner.Inventory; Inv!=None; Inv=Inv.Inventory)
   {
      if (Inv.bIsAnArmor)
      {
         if (Inv.IsA('Antacid'))
            bShieldbelt = true;
         else if (Inv.IsA('Apron'))
         {
            ThighAmount += Inv.Charge;
            bThighArmor = true;
         }
         else
         {
            bChestArmor = true;
            ChestAmount += Inv.Charge;
         }
         ArmorAmount += Inv.Charge;
      }
      else if (Inv.IsA('UT_JumpBoots'))
         bJumpBoots = true;
      else
      {
         i++;
         if (i > 100)
            break; // can occasionally get temporary loops in netplay
      }
   }

   if (!bHideStatus)
   {
      TPOwner = TournamentPlayer(PawnOwner);
      if (Canvas.ClipX < 400)
         bHasDoll = false;
      else if (TPOwner != None)
      {
         Doll = TPOwner.StatusDoll;
         DollBelt = TPOwner.StatusBelt;
         bHasDoll = true;
      }
      else
      {
         BotOwner = Bot(PawnOwner);
         if (BotOwner != None)
         {
            Doll = BotOwner.StatusDoll;
            DollBelt = BotOwner.StatusBelt;
            bHasDoll = true;
         }
      }
      if (bHasDoll)
      {
         Canvas.Style = ERenderStyle.STY_Translucent;
         StatScale = Scale * StatusScale;
         X = Canvas.ClipX - 128 * StatScale;
         Canvas.SetPos(X, 0);
         if (PawnOwner.DamageScaling > 2.0)
            Canvas.DrawColor = PurpleColor;
         else
            Canvas.DrawColor = HUDColor;
         Canvas.DrawTile(Doll, 128*StatScale, 256*StatScale, 0, 0, 128.0, 256.0);
         Canvas.DrawColor = HUDColor;
         if (bShieldBelt)
         {
            Canvas.DrawColor = BaseColor;
            Canvas.DrawColor.B = 0;
            Canvas.SetPos(X, 0);
            Canvas.DrawIcon(DollBelt, StatScale);
         }
         if (bChestArmor)
         {
            ChestAmount = FMin(0.01 * ChestAmount,1);
            Canvas.DrawColor = HUDColor * ChestAmount;
            Canvas.SetPos(X, 0);
            Canvas.DrawTile(Doll, 128*StatScale, 64*StatScale, 128, 0, 128, 64);
         }
         if (bThighArmor)
         {
            ThighAmount = FMin(0.02 * ThighAmount,1);
            Canvas.DrawColor = HUDColor * ThighAmount;
            Canvas.SetPos(X, 64*StatScale);
            Canvas.DrawTile(Doll, 128*StatScale, 64*StatScale, 128, 64, 128, 64);
         }
         if (bJumpBoots)
         {
            Canvas.DrawColor = HUDColor;
            Canvas.SetPos(X, 128*StatScale);
            Canvas.DrawTile(Doll, 128*StatScale, 64*StatScale, 128, 128, 128, 64);
         }
         Canvas.Style = Style;
         if ((PawnOwner == PlayerOwner) && Level.bHighDetailMode && !Level.bDropDetail)
         {
            for (i=0; i<4; i++)
            {
               DamageTime = Level.TimeSeconds - HitTime[i];
               if (DamageTime < 1)
               {
                  Canvas.SetPos(X + HitPos[i].X * StatScale, HitPos[i].Y * StatScale);
                  if ((HUDColor.G > 100) || (HUDColor.B > 100))
                     Canvas.DrawColor = RedColor;
                  else
                     Canvas.DrawColor = (WhiteColor - HudColor) * FMin(1, 2 * DamageTime);
                     Canvas.DrawColor.R = 255 * FMin(1, 2 * DamageTime);
                     Canvas.DrawTile(Texture'BotPack.HudElements1', StatScale * HitDamage[i] * 25, StatScale * HitDamage[i] * 64, 0, 64, 25.0, 64.0);
               }
            }
         }
      }
   }
   Canvas.DrawColor = HUDColor;
   if (bHideStatus && bHideAllWeapons)
   {
      X = 0.5 * Canvas.ClipX;
      Y = Canvas.ClipY - 64 * Scale;
   }
   else
   {
      X = Canvas.ClipX - 128 * StatScale - 140 * Scale;
      Y = 64 * Scale;
   }
   Canvas.SetPos(X,Y);
   if (PawnOwner.Health < 50)
   {
      H1 = 1.5 * TutIconBlink;
      H2 = 1 - H1;
      Canvas.DrawColor = WhiteColor * H2 + (HUDColor - WhiteColor) * H1;
   }
   else
      Canvas.DrawColor = HUDColor;
   Canvas.DrawTile(Texture'BotPack.HudElements1', 128*Scale, 64*Scale, 128, 128, 128.0, 64.0);

   if (PawnOwner.Health < 50)
   {
      H1 = 1.5 * TutIconBlink;
      H2 = 1 - H1;
      Canvas.DrawColor = Canvas.DrawColor * H2 + (WhiteColor - Canvas.DrawColor) * H1;
   }
   else
      Canvas.DrawColor = WhiteColor;

   DrawBigNum(Canvas, Max(0, PawnOwner.Health), X + 4 * Scale, Y + 16 * Scale, 1);

   Canvas.DrawColor = HUDColor;
   if (bHideStatus && bHideAllWeapons)
   {
      X = 0.5 * Canvas.ClipX - 128 * Scale;
      Y = Canvas.ClipY - 64 * Scale;
   }
   else
   {
      X = Canvas.ClipX - 128 * StatScale - 140 * Scale;
      Y = 0;
   }
   Canvas.SetPos(X, Y);
   Canvas.DrawTile(Texture'BotPack.HudElements1', 128*Scale, 64*Scale, 0, 192, 128.0, 64.0);
   if (bHideStatus && bShieldBelt)
      Canvas.DrawColor = GoldColor;
   else
      Canvas.DrawColor = WhiteColor;
   DrawBigNum(Canvas, Min(150,ArmorAmount), X + 4 * Scale, Y + 16 * Scale, 1);
    
	//peppermill effect
   if (PlayerIsPeppered())
   {
      Canvas.Style = ERenderStyle.STY_Translucent;

      Canvas.SetPos(0.0,0.0);
      Canvas.DrawRect(Texture'poisonmask', Canvas.ClipX, Canvas.ClipY);
   	//first row
      Canvas.SetPos(0.0,0.0);
      Canvas.DrawRect(Texture'FoodFight.Addons.Peppermill.PepperHUD', Canvas.ClipX / 4.0 , Canvas.ClipY / 3.0);
      Canvas.SetPos(Canvas.ClipX / 4.0 ,0.0);
      Canvas.DrawRect(Texture'FoodFight.Addons.Peppermill.PepperHUD', Canvas.ClipX / 4.0 , Canvas.ClipY / 3.0);
      Canvas.SetPos(2*(Canvas.ClipX / 4.0) ,0.0);
      Canvas.DrawRect(Texture'FoodFight.Addons.Peppermill.PepperHUD', Canvas.ClipX / 4.0 , Canvas.ClipY / 3.0);
      Canvas.SetPos(3*(Canvas.ClipX / 4.0) ,0.0);
      Canvas.DrawRect(Texture'FoodFight.Addons.Peppermill.PepperHUD', Canvas.ClipX / 4.0 , Canvas.ClipY / 3.0);
      Canvas.SetPos(0.0, Canvas.ClipY / 3.0);
      //second row
      Canvas.DrawRect(Texture'FoodFight.Addons.Peppermill.PepperHUD', Canvas.ClipX / 4.0 , Canvas.ClipY / 3.0);
      Canvas.SetPos(Canvas.ClipX / 4.0 ,Canvas.ClipY / 3.0);
      Canvas.DrawRect(Texture'FoodFight.Addons.Peppermill.PepperHUD', Canvas.ClipX / 4.0 , Canvas.ClipY / 3.0);
      Canvas.SetPos(2*(Canvas.ClipX / 4.0) ,Canvas.ClipY / 3.0);
      Canvas.DrawRect(Texture'FoodFight.Addons.Peppermill.PepperHUD', Canvas.ClipX / 4.0 , Canvas.ClipY / 3.0);
      Canvas.SetPos(3*(Canvas.ClipX / 4.0) , Canvas.ClipY / 3.0);
      Canvas.DrawRect(Texture'FoodFight.Addons.Peppermill.PepperHUD', Canvas.ClipX / 4.0 , Canvas.ClipY / 3.0);
      Canvas.SetPos(0.0, 2*(Canvas.ClipY / 3.0));
      //third row
      Canvas.DrawRect(Texture'FoodFight.Addons.Peppermill.PepperHUD', Canvas.ClipX / 4.0 , Canvas.ClipY / 3.0);
      Canvas.SetPos(Canvas.ClipX / 4.0 , 2*(Canvas.ClipY / 3.0));
      Canvas.DrawRect(Texture'FoodFight.Addons.Peppermill.PepperHUD', Canvas.ClipX / 4.0 , Canvas.ClipY / 3.0);
      Canvas.SetPos(2*(Canvas.ClipX / 4.0) , 2*(Canvas.ClipY / 3.0));
      Canvas.DrawRect(Texture'FoodFight.Addons.Peppermill.PepperHUD', Canvas.ClipX / 4.0 , Canvas.ClipY / 3.0);
      Canvas.SetPos(3*(Canvas.ClipX / 4.0) , 2*(Canvas.ClipY / 3.0));
      Canvas.DrawRect(Texture'FoodFight.Addons.Peppermill.PepperHUD', Canvas.ClipX / 4.0 , Canvas.ClipY / 3.0);
   }
}

simulated function bool PlayerIsPeppered()
{
   local PepperEffect PE;

   PE = PepperEffect(PlayerOwner.FindInventoryType(class'PepperEffect'));
   if ((PE == None) || !(PE.bPepperActive))
      return false;
   else
      return true;
}

defaultproperties
{
   ServerInfoClass=class'FoodFight.ServerInfoBurgerCTF'
}
