//
//this class checks the map for burgerspawns and creates them if needed
class BurgerCTFMapSetup extends Info config;

var float saveDist[32];
var int NumRetries;
var InventorySpot FlagLocation;
var config bool bUseFlagPreConfig;

function RetryWithout(float Dist)
{
	NumRetries++;
	if (NumRetries > ArrayCount(saveDist))
	{
		log("*** Burger placing FAILED! No burger in game! ***");
		BroadcastMessage("*** NO BURGER IN GAME ***");
		return;
	}
	saveDist[NumRetries - 1] = Dist;
	PreBeginPlay();
}

function Tick(float DeltaTime)
{
	// clear flag location
	if (FlagLocation != None && FlagLocation.MarkedItem != None && !FlagLocation.MarkedItem.bDeleteMe)
	{
		FlagLocation.MarkedItem.Destroy();
	}
}

function bool CheckDistance(float Dist)
{
	local int i;
	
	for (i = 0; i < NumRetries; i++)
		if (saveDist[i] == Dist)
			return false;
	return true;
}

function PreBeginPlay()
{
	local FlagBase Flag, RedFlag, BlueFlag;
	local NavigationPoint NP, GoodSpot;
	local float minDist, curDist;
	local vector AdjustedLocation;
	
	ForEach AllActors(class'Botpack.FlagBase', Flag) 
	{
		if (Flag.IsA('NeutralFlagBase'))
			return;	// the map has a burger
		if (Flag.Team == 0)
			RedFlag = Flag;
		if (Flag.Team == 1)
			BlueFlag = Flag;
	}
	
	minDist = VSize(RedFlag.Location - BlueFlag.Location) / 5;
	
	if (bUseFlagPreConfig && KnownMap()) return;
	
	// no burger on this map and map has no known pre-configuration,
	// so we need to find a good place to spawn it
	ForEach AllActors(class'Engine.NavigationPoint', NP) 
	{
		curDist = Abs(VSize(NP.Location - RedFlag.Location) - VSize(NP.Location - BlueFlag.Location));
		if (curDist < minDist && CheckDistance(curDist) && (NP.IsA('PathNode')
				|| NP.IsA('AlternatePath') || NP.IsA('InventorySpot')) && (!NP.Region.Zone.bWaterZone))
			if (Rand(3) > 0)
			{	// randomize the flag's position
				GoodSpot = NP;
				minDist = curDist;
			}
	}
	
	if (GoodSpot == None)
		warn("Couldn't find good location for burger.");
	else 
	{
		log("Good location for burger found:"@GoodSpot@"at"@GoodSpot.Location);
		if (!PlaceAt(GoodSpot.Location)) 
		{
			RetryWithout(minDist);	// try again, but exclude this single spot by distance
			return;
		}
		FlagLocation = InventorySpot(GoodSpot);
	}
}

function bool PlaceAt(vector DesiredLocation)
{
	local vector HitLoc, HitNormal, EndLoc;
	local NeutralFlagBase SpawnCheck;
	
	EndLoc = DesiredLocation;
	EndLoc.Z -= 1000.0;
	if (!FastTrace(EndLoc, DesiredLocation)) 
	{
		Trace(HitLoc, HitNormal, EndLoc, DesiredLocation, False);
		HitLoc.Z += 70.0;
	}
	else
		HitLoc = DesiredLocation;
	
	SpawnCheck = Spawn(class'NeutralFlagBase',,, HitLoc);
	if (SpawnCheck == None) 
	{
		warn("Couldn't place burger.");
		return false;
	}
	return true;
}

function bool KnownMap()
{
	local Actor A;
	local name ActorName;
	local NeutralFlagBase Check;
	local NeutralFlagInfo FlagInfo;
	local float r;
	
	// get one of the flag locations specified by map maker
	ForEach AllActors(class'NeutralFlagInfo', FlagInfo)
		ActorName = FlagInfo.FlagLocations[Rand(FlagInfo.NumLocations)];
	
	if (ActorName == '')
		switch (Level.Title) 
		{
		case "EternalCaves":
			if (FRand() < 0.5)
				ActorName = 'ZoneInfo2';	// lower passage (between red and blue rl room)
			else
				ActorName = 'Light33';	// upper passage (minigun room)
			break;
		case "November":
			r = FRand();
			if (r < 0.6)
				ActorName = 'PathNode59';	// on sub
			else if (r < 0.9)
				ActorName = 'PathNode49';	// on sub
			else
				ActorName = 'PathNode50';	// on sub
			break;
		case "The Iron Gauntlet":
			if (FRand() < 0.5)
				ActorName = 'PathNode179';
			else
				ActorName = 'Light29';
			break;
		case "The Last Command":
			ActorName = 'Light71';	// near shock rifle
			break;
		case "Coret Facility":
			r = FRand();
			if (r < 0.35)
				ActorName = 'PathNode23';
			else if (r < 0.7)
				ActorName = 'PathNode18';
			else
				ActorName = 'InventorySpot68';	// damage amplifier
			break;
		case "Dreary Outpost":
			ActorName = 'PathNode76';
			break;
		case "Facing Worlds":
			ActorName = 'InventorySpot126';	// health pack
			break;
		case "Facing Worlds ][":
			ActorName = 'InventorySpot63';	// health pack
			break;
		case "Facing Worlds Special Edition":
			ActorName = 'InventorySpot479';	// health pack
			break;
		case "Final Face":
			ActorName = 'InventorySpot75';	// health pack
			break;
		case "Hydro Bases":
			if (FRand() < 0.5)
				ActorName = 'PathNode37';
			else
				ActorName = 'PathNode110';
			break;
		case "Noxion Base":
			ActorName = 'PathNode122';
			break;
		case "Orbital Station 12":
			r = FRand();
			if (r < 0.35)
				ActorName = 'PathNode62';	// left from shieldbelt (view from red side)
			else if (r < 0.7)
				ActorName = 'PathNode262';	// right from shieldbelt (view from red side)
			else
				ActorName = 'InventorySpot241';	// shield belt
			break;
		case "Hall of Giants":
			if (FRand() < 0.5)
				ActorName = 'InventorySpot108';	// health pack
			else 
				ActorName = 'InventorySpot109';	// shield belt
			break;
		case "High Towers":
			if (FRand() < 0.5)
				ActorName = 'PathNode247';	// lower bridge
			else
				ActorName = 'Trigger5';	// on crane arm
			break;
		case "LavaGiant":
			r = FRand();
			if (r < 0.2)
				ActorName = 'PathNode43';	// near central rocket launcher
			else if (r < 0.4)
				ActorName = 'PathNode132';	// at central rocket launcher
			else if (r < 0.6)
				ActorName = 'PathNode135';	// at redeemer passage
			else if (r < 0.8)
				ActorName = 'PathNode57';	// at redeemer window
			else
				ActorName = 'InventorySpot47';	// shield belt
			break;
		case "Diamond Sword":
			r = FRand();
			if (r < 0.33)
				ActorName = 'InventorySpot19';	// central rocket launcher
			else if (r < 0.67)
				ActorName = 'InventorySpot50';	// lower central flak cannon
			else
				ActorName = 'InventorySpot52';	// central health pack
			break;
		default:
			return false;
			break;
		}

	// the level is known, search for the specified object
	ForEach AllActors(class'Engine.Actor', A)
		if (A.Name == ActorName) 
		{
			log("Specified location for burger is"@ActorName@"("$A.Location$")");
			if (!PlaceAt(A.Location)) 
			{
				log("Spawning at specified location failed, looking for other good location.");
				return false;
			}
			FlagLocation = InventorySpot(A);
			return true;
		}
	
	log("Specified location ("$ActorName$") not found, looking for other good location.");
	return false;
}

defaultproperties
{
   bUseFlagPreConfig=True
}
