//
class CarrotProjectile extends projectile config(FoodFight);

#exec MESH IMPORT MESH=CarrotProjectile ANIVFILE=MODELS\CarrotProjectile_a.3d DATAFILE=MODELS\CarrotProjectile_d.3d X=0 Y=0 Z=0
#exec MESH ORIGIN MESH=CarrotProjectile X=0 Y=0 Z=0 YAW=-64 PITCH=0 ROLL=0

#exec MESH SEQUENCE 	MESH=CarrotProjectile 	SEQ=All 		STARTFRAME=0 NUMFRAMES=1
#exec MESH SEQUENCE 	MESH=CarrotProjectile 	SEQ=Wing 	STARTFRAME=0 NUMFRAMES=1
#exec MESH SEQUENCE 	MESH=CarrotProjectile 	SEQ=WingIn 	STARTFRAME=0 NUMFRAMES=1
#exec MESH LODPARAMS MESH=CarrotProjectile 	STRENGTH=0

#exec MESHMAP NEW MESHMAP=CarrotProjectile MESH=CarrotProjectile
#exec MESHMAP SCALE MESHMAP=CarrotProjectile X=0.1 Y=0.1 Z=0.2

#exec MESHMAP SETTEXTURE MESHMAP=CarrotProjectile NUM=1 TEXTURE=CarroteerUVmap

#exec AUDIO IMPORT NAME="carrotthud" 	FILE=Sounds\carrotthud.wav
#exec AUDIO IMPORT NAME="BugsA" 			FILE=Sounds\BugsA.wav
#exec AUDIO IMPORT NAME="BugsB" 			FILE=Sounds\BugsB.wav
#exec AUDIO IMPORT NAME="BugsC" 			FILE=Sounds\BugsC.wav
#exec AUDIO IMPORT NAME="BugsD" 			FILE=Sounds\BugsD.wav
#exec AUDIO IMPORT NAME="BugsE" 			FILE=Sounds\BugsE.wav

#exec AUDIO IMPORT NAME="CarrotLaunch" 			FILE=Sounds\CarrotLaunch.wav

var() float ShallowAngle; //ShallowAngle= 0.35 
var() bool CanHitInstigator;
var() float HitInstigatorTimeOut;

var bool bHitWater;
var redeemertrail trail;

//Seeking code
var float SmokeRate, TurnRate;
var Actor Seeking;
var vector InitialDir;
var config int MissileSpeed;
var config int TurnRadius;

//need to replicate this stuff so everyone sees the same things
replication
{
   reliable if( Role==ROLE_Authority )
      Seeking, InitialDir, MissileSpeed, TurnRadius;
}

simulated function PostBeginPlay()
{
   SmokeRate = 0.3;
   SetTimer(0.3,false);
}

simulated function Destroyed()
{
   if ( Trail != None )
      Trail.Destroy();
   Super.Destroyed();
}

simulated function Timer()
{
   local vector x, y, z, SeekingDir;
   local float bestDist, bestAim, MagnitudeVel;
   local actor other;
   local Pawn p;
   local FlyingLeaf b;

   Speed=MissileSpeed;

   if ( InitialDir == vect(0,0,0) )
      InitialDir = Normal(Velocity);

   If (Seeking==None)
      bestAim = 1.00; //0.93

   if (Pawn(Seeking) != None && (Pawn(Seeking).Health <= 0 || Pawn(Seeking).bHidden))
      Seeking = None;

   If (Seeking != None  && Seeking != Instigator)
   {
   	//Carrot has locked on someone, call the message for it
   	if ((Pawn(Seeking) != None) && (Pawn(Seeking).IsA('PlayerPawn')))
      	Pawn(Seeking).ReceiveLocalizedMessage( class'LockedMessage',, Pawn(Seeking).PlayerReplicationInfo, Pawn(Seeking).PlayerReplicationInfo );

		//if locked it will spawn a rocket flame tail
      if (Trail == None)
        Trail = Spawn(class'RedeemerTrail',self);
      if (AmbientSound == None)
      	AmbientSound = sound'CarrotLaunch';

		//seeking code
      SeekingDir = Normal(Seeking.Location - Location);
      if ((SeekingDir Dot InitialDir) > 0)
      {
         MagnitudeVel = VSize(Velocity);
         SeekingDir = Normal(SeekingDir * 1.2 * MagnitudeVel + Velocity);
         Velocity =  MagnitudeVel * SeekingDir;
         Acceleration = TurnRadius * SeekingDir;
         SetRotation(rotator(Velocity));
      }
   }
   if (Region.Zone.bWaterZone || (Level.NetMode == NM_DedicatedServer))
   {
      SetTimer(SmokeRate, false);
      Return;
   }
   if (Level.bHighDetailMode)
   {
      SmokeRate = 152/Speed;
		b = Spawn(class'FlyingLeaf');
      b.RemoteRole = ROLE_None;
   }
   else
   {
      SmokeRate = 0.15;
      b = Spawn(class'FlyingLeaf');
      b.RemoteRole = ROLE_None;
   }

   SetTimer(SmokeRate, false);
}

auto state Flying
{
   simulated function ZoneChange(Zoneinfo NewZone)
   {
      local waterring w;
        
      if (!NewZone.bWaterZone || bHitWater) Return;

      bHitWater = True;
      if (Level.NetMode != NM_DedicatedServer)
      {
         w = Spawn(class'WaterRing',,,,rot(16384,0,0));
         w.DrawScale = 0.2;
         w.RemoteRole = ROLE_None;
         PlayAnim( 'Wing', 3.0 );
      }
      Velocity=0.6*Velocity;
   }

   simulated function ProcessTouch (Actor Other, Vector HitLocation)
   {
   	local int randm;

		//check to prevent shooting yourself
      if ((!CanHitInstigator || (CanHitInstigator && (Default.LifeSpan - LifeSpan) < HitInstigatorTimeOut)) && Other == Instigator)
			return;

		//won't spawn a stuck carrot
   	if ((Other != instigator) && (Other.IsA('Decoration')))
         AltExplode(HitLocation,Normal(HitLocation-Other.Location));
      else

		//we hit a player, maybe they get taunted?
		if ((Other != instigator) && (Other.IsA('Pawn')))
		{
         AltExplode(HitLocation,Normal(HitLocation-Other.Location));
         if (Other != none)
         {
         	if (FRand() < 0.1)
         	{
         		randm = Rand(5);
      			switch(randm)
      			{
         			case 0:
            		Playsound(sound'BugsA');
            		break;
         			case 1:
            		Playsound(sound'BugsB');
            		break;
         			case 2:
            		Playsound(sound'BugsC');
            		break;
         			case 3:
            		Playsound(sound'BugsD');
            		break;
         			case 4:
            		Playsound(sound'BugsE');
            		break;
            	}
            }
			}		
      }
		else if ((Other != instigator) && !Other.IsA('Projectile'))
         Explode(HitLocation,Normal(HitLocation-Other.Location));
   }

   simulated function HitWall (vector HitNormal, actor Wall)
	{
		if (Mover(Wall) != None)
		{
			if (Role == ROLE_Authority)
			{
				if (Mover(Wall).bDamageTriggered)
					Wall.TakeDamage( Damage, instigator, Location, MomentumTransfer * Normal(Velocity), '');
				AltExplode(Location + ExploWallOut * HitNormal, HitNormal);
				MakeNoise(1.0);
			}
		}
		else
		{
			if (Role == ROLE_Authority)
        		MakeNoise(1.0);

			if (((Normal(-Velocity) dot HitNormal) < ShallowAngle) ||  (Trail != None))
			{
    			Velocity -= (2 * (Velocity dot HitNormal) * HitNormal);
    			SetRotation(rotator(Velocity));
    			AltExplode(Location + ExploWallOut * HitNormal, HitNormal);
    			if ((ExplosionDecal != None) && (Level.NetMode != NM_DedicatedServer))
        			Spawn(ExplosionDecal,self,,Location, rotator(HitNormal));
			}
			else
			{
				Explode(Location + ExploWallOut * HitNormal, HitNormal);
				PlaySound(ImpactSound, SLOT_Misc, 2.0);
				if (Role == ROLE_Authority)
    	   		Spawn(class'CarrotStuck',,,Location + HitNormal*16);
			}
		}
	}

   function BlowUp(vector HitLocation)
   {  //Remove damage
      MakeNoise(1.0);
   }

  	// This is to restore the explosion/destruction if the carrot hits at too much of an angle
   function AltBlowUp(vector HitLocation)
   {
      HurtRadius(Damage,220.0, MyDamageType, MomentumTransfer, HitLocation );
      MakeNoise(1.0);
   }

   simulated function Explode(vector HitLocation, vector HitNormal)
   {
      BlowUp(HitLocation);
     	Destroy();
   }

	// This is to restore the explosion/destruction if the carrot hits at too much of an angle
   simulated function AltExplode(vector HitLocation, vector HitNormal)
   {
      local UT_SpriteBallExplosion s;
      local int i;
   	local CarrotFragments b;
   	local vector start;

      AltBlowUp(HitLocation);
		
		start = Location + 10 * HitNormal;
		Spawn( class'ut_FlameExplosion',,,Start);
    	Spawn( class 'CarrotChunk2',, '', Start);
    	Spawn( class 'CarrotChunk3',, '', Start);
    	Spawn( class 'CarrotChunk4',, '', Start);
    	Spawn( class 'CarrotChunk1',, '', Start);
    	Spawn( class 'CarrotChunk2',, '', Start);

     	Destroy();
   }

   function BeginState()
   {
      local vector InitialDir,x,y,z;
      local vector SeekingDir;
      local float MagnitudeVel;
      local float bestDist, bestAim;
      local actor other;
      local Pawn p;
      
      Speed=MissileSpeed;

      initialDir = vector(Rotation);
      if (Role == ROLE_Authority)
         Velocity = speed*initialDir;
         
      Acceleration = initialDir*50;
      
      If (Seeking==None)
      {
         bestAim = 1.00;//0.93
         Seeking = getTarget( 30000, Instigator.Location, vector(Rotation), Class'Pawn', 0.9995, True);
      }
   }
}

//Seeking code
function actor getTarget( float Radius, vector Loc, vector Dir, optional class<Actor> classType, optional float periphericalFactor, optional bool getEnemyOnly)
{
   local Actor A, closerA;

   if (classType == None) classType = Class'Actor';

   ForEach VisibleCollidingActors(classType, A, FMax(Radius, 1.0))
   {
      if ((!getEnemyOnly || (getEnemyOnly && !isFriendlyPawn(Pawn(A)))) && (Normal(A.Location - Loc) dot Normal(Dir)) >= periphericalFactor
	  && (closerA == None || (VSize(A.Location - Loc) < VSize(closerA.Location - Loc))))
         closerA = A;
   }
   return closerA;
}

//won't target friendly
function bool isFriendlyPawn( Pawn P)
{
   if (P == None || Instigator == None)
      return False;
   if (P == Instigator)
      return True;
   if (ScriptedPawn(P) != None)
      return False;
   if (TeamGamePlus(Level.Game) != None && P.PlayerReplicationInfo != None && Instigator.PlayerReplicationInfo != None)
      return (P.PlayerReplicationInfo.Team == Instigator.PlayerReplicationInfo.Team);
   return False;
}

defaultproperties
{
	speed=MissileSpeed
	TurnRadius=100.00
	bNetTemporary=False
	bProjTarget=True
	HitInstigatorTimeOut=0.500000
	CanHitInstigator=True
   ExplosionDecal=Class'CarrotBlastMark'
   ImpactSound=Sound'carrotthud'
	ShallowAngle=0.45
   MyDamageType=CarrotDeath
   speed=1400.000000
   MaxSpeed=1400.000000
   Damage=80.000000
   MomentumTransfer=50000
   SpawnSound=Sound'UnrealShare.Eightball.Ignite'
   MiscSound=Sound'UnrealShare.General.Explg02'
   RemoteRole=ROLE_SimulatedProxy
   LifeSpan=12.000000
   AnimSequence=WingIn
   Mesh=CarrotProjectile
   DrawScale=0.75000
   AmbientGlow=64
   bUnlit=True
   bMeshCurvy=False
   SoundRadius=50
   SoundVolume=255
   SoundPitch=100
   AmbientSound=none
   LightType=LT_Steady
   LightEffect=LE_NonIncidence
   LightBrightness=100
   LightHue=28
   LightSaturation=0
   LightRadius=6
   bBounce=True
   bFixedRotationDir=True
   RotationRate=(Roll=10000)
   DesiredRotation=(Pitch=12000,Yaw=5666,Roll=2334)
}
