//
//base class for the fart. This was all done by shrimp, I just made some minor adjustments
//most of these options I did not use but they are there if you want to use them
class EffectParticleBase expands Effects;

// User-defined options
var(ParticleSystem) bool bAutoStart;					// Starts the Particles upon game startup
var(ParticleSystem) bool bFadeOut;						// Particles will fade out
var(ParticleSystem) bool bShrink;						// Particles will shrink for the duration of their lifspan
var(ParticleSystem) bool bGrow;							// Particles will grow for the duration of their lifspan
var(ParticleSystem) bool bRising;						// Particles rise like fire/smoke
var(ParticleSystem) bool bExploding;					// Particles burst outwards
var(ParticleSystem) bool bConstant;						// Ignores NumParticles and makes particles forever
var(ParticleSystem) float ParticleLife;				// LifeSpan of Particles
var(ParticleSystem) float ParticleLifeVar;			// Particles LifeSpan variation
var(ParticleSystem) float ParticleSize;				// DrawScale of Particles
var(ParticleSystem) float ParticleSizeVar;			// Particles DrawScale variation
var(ParticleSystem) float GrowRate;						// Rate the particles will grow at
var(ParticleSystem) int ParticleOffset;				// Random offset location for Particles
var(ParticleSystem) int NumParticles;					// Number of Particles to spawn
var(ParticleSystem) int RisingRate;						// Speed at which Particles rise if bRising
var(ParticleSystem) int ExplodeSize;					// Area bExploding Particles will cover
var(ParticleSystem) int SprayHeight;					// Height to send bExploding Particles flying
var(ParticleSystem) int SimultaneousParticles;		// Number of Particles to spawn at once
var(ParticleSystem) texture ParticleTexture;			// Texture for the particles
var(ParticleSystem) enum EParticleDrawStyle
{
	PDS_Masked,
	PDS_Translucent,
	PDS_Modulated,
} ParticleDrawStyle;
var(ParticleSystem) enum EParticleType
{
	PT_Rising,
	PT_Exploding,
} ParticleType;

var bool bActive;
var Actor PartOwner;

simulated function PostBeginPlay()
{
	local Bot B;

	super.PostBeginPlay();

	if (!bAutoStart)
		Disable('Tick');
}

simulated function Trigger(actor Other, pawn EventInstigator)
{
	if (bActive)
	{
		Disable('Tick');
		bActive = False;
	}
	else
		Enable('Tick');
}

simulated function Tick(float DeltaTime)
{
	local EffectParticle P;
	local int i;

	bActive = True;
	
	for (i = 1; i <= SimultaneousParticles; i++) 
	{
		if (!bConstant)
			NumParticles --;
		P = Spawn(class'EffectParticle',Owner,,Location+ParticleOffset*VRand());
		if ( P != None )
		{
			P.RemoteRole = ROLE_SimulatedProxy;
			P.DrawScale = ParticleSize + (FRand() * ParticleSizeVar);
			P.Texture = ParticleTexture;
			P.bFadeOut = bFadeOut;
			P.bShrink = bShrink;
			P.bGrow = bGrow;
			P.GrowRate = GrowRate;
			switch (ParticleDrawStyle)
			{
				case PDS_Masked :
					P.Style = STY_Masked;
					break;
				case PDS_Modulated :
					P.Style = STY_Modulated;
					break;
				case PDS_Translucent :
					P.Style = STY_Translucent;
					break;
				default:
					P.Style = STY_Translucent;
					break;
			}
			SetParticleBehaviour(P);
			P = None;
		}

		if ((NumParticles <= 0) && !bConstant)
		{
			bActive = False;
			Disable('Tick');
		}
	}
}

simulated function SetParticleBehaviour(EffectParticle P)
{
	local vector TempVect;

	if (ParticleType == PT_Rising)
	{
		P.SetPhysics(PHYS_Projectile);
		P.Velocity = Vector(Rotation) * (-13 - Rand(15)); // Vect(0,0,1) * RisingRate;
	}
	else if (ParticleType == PT_Exploding)
	{
		P.SetPhysics(PHYS_Falling);
		P.Velocity = CalcVelocity(TempVect, ExplodeSize);
	}
}

simulated function vector CalcVelocity(vector Momentum, float ExplosionSize)
{
	local vector TempVect;

	TempVect = VRand()*(ExplosionSize+FRand()*150+100+VSize(Momentum)/80);
	TempVect.Z += Rand(SprayHeight) + (SprayHeight * 2);

	return TempVect;
}

defaultproperties
{
   bAutoStart=True
   bFadeOut=True
   bGrow=True
   bRising=True
   ParticleLife=8.00
   ParticleLifeVar=4.00
   ParticleSize=0.30
   ParticleSizeVar=0.50
   GrowRate=0.00
   ParticleOffset=15
   Numparticles=12
   RisingRate=5
   ExplodeSize=300
   SprayHeight=200
   SimultaneousParticles=1
   ParticleTexture=Texture'Fart'
   ParticleDrawStyle=PDS_Translucent
   bHidden=True
   Physics=PHYS_Trailer
   DrawType=DT_Sprite
   Texture=Texture'Fart'
}