//
//base mod class
class FFMod expands Mutator config(FoodFight);

var config string DMGameName, DMLeftMessage, DMEnteredMessage;
var bool bInitialized;

//FoodFight Melee match
var config bool GiveAllWeapons;
var config bool UseLunchBox;

var() PlayerPawn PlayerOwner;

//added for compatibility to Higor's XC_GameEngine
//this will allow easy usage
native(1718) final function bool AddToPackageMap( optional string PkgName); //Dummy declaration, compiler won't link to DLL

function PreBeginPlay ()
{
	Super.PreBeginPlay();
	SaveConfig();
	
	if (Level.Game.Class==Class'DeathMatchPlus')
	{
		Level.Game.ScoreBoardType = Class'FFScoreBoard';
		Level.Game.HUDType = Class'FFHUD';
		Level.Game.GameName = DMGameName;
		Level.Game.LeftMessage = DMLeftMessage;
		Level.Game.EnteredMessage = DMEnteredMessage;
	}
}

function PostBeginPlay()
{
   if (bInitialized)
		return;
	bInitialized = True;

	//for Higor's XC_Engine, does nothing otherwise
	if (int(ConsoleCommand("Get ini:Engine.Engine.GameEngine XC_Version")) >= 11 || int(ConsoleCommand("XC_Engine")) >= 11)
      AddToPackageMap(); //Conditional above is there to prevent invalid opcode crashes, makes this code 'multiplatform'

   if (NextMutator != None)
		NextMutator.PostBeginPlay();
}

function bool HandlePickupQuery(Pawn Other, Inventory item, out byte bAllowPickup)
{
   local bool bItemFound;
   local inventory I;

   bItemFound = false;
   if (Item != None)
   {
      for (I=Other.Inventory; I!=None; I=I.Inventory)
      {
      //I nulled these checks out but you can restore them if you want. They are used to
      //prevent multiple pickups of the same item
      /*
	 		if (Item.bIsAnArmor && I.bIsAnArmor && Item.Class == I.Class && I.Charge >= I.Default.Charge)
	 		{
	    		bItemFound = true;
         }
         else if (Item.IsA('Antacid') && I.IsA('Antacid') && I.Charge < I.Default.Charge )
         {
	    		bItemFound = false;
	 		}
	 		else if (Item.IsA('CoffeePickup') && I.IsA('CoffeePickup'))
	 		{
	    		bItemFound = true;
	 		}
	 		else if (Item.IsA('PepperMill') && I.IsA('PepperMill'))
	 		{
	    		bItemFound = true;
	 		}
	 		else if (Item.IsA('Armor') && I.IsA('Armor'))
	 		{
            bItemFound = true;
	 		}

	 		else 
			 */
			//this is used to prevent thrown weapons from being picked back up. The reason why is that
			//because of the reloading animations it was possible to cheat the remaining ammo in weapons
			//by tossing them at the right time. Instead of recoding everything I just added this.
			if (Item.IsA('FFWeapon') && (FFWeapon(Item).bTossedOut))
	 		{
	 			//yes this weapon was tossed out
	 			bItemFound = true;
	 			//warn the player it cannot be taken
	 			if (!Other.IsA('Bot'))
	 				Other.ReceiveLocalizedMessage(class'NoPickupMessage');
	 		}
      }
      //stop from picking up
      if (bItemFound)
	 		return true;
   }
   else
		return false;

   if (NextMutator != None)
      return NextMutator.HandlePickupQuery(Other, item, bAllowPickup);
}

//A quick word on replacing items: it's best to do a perfect class check instead of an 'IsA'
//type check. This will stop runaway replacements since the replaced item must exactly match 
//the class you declare.
function bool CheckReplacement(Actor Other, out byte bSuperRelevant)
{
	local int random, randomA;

   if(Other.IsA('Pickup'))
   {
      Pickup(Other).bAutoActivate=True;
   }
//Weapon replacements 
	if(Other.Class==Class'Chainsaw')
   {
      ReplaceWith(Other,"FoodFight.CanOpener");
      return false;
   }
   if(Other.Class==Class'Enforcer')
   {
      ReplaceWith(Other,"FoodFight.CanOpener");
      return false;
   }
   if(Other.Class==Class'ImpactHammer')
   {
   	//Don't replace the hammer, it causes problems online with the bots
      return false;
   }
   if(Other.Class==Class'Minigun2')
   {
      ReplaceWith(Other,"FoodFight.PeanutGun");
      return false;
   }
   if(Other.Class==Class'PulseGun')
   {
      ReplaceWith(Other,"FoodFight.SweetCherryCan");
      return false;
   }
   if(Other.Class==Class'Ripper')
   {
      ReplaceWith(Other,"FoodFight.Carroteer");
      return false;
   }
   if(Other.Class==Class'ShockRifle')
   {
      ReplaceWith(Other,"FoodFight.PapriCannonGun");
      return false;
   }
   if(Other.Class==Class'SuperShockRifle')
   {
   	if(UseLunchBox)
   	{
   	   ReplaceWith(Other,"FoodFight.lunchbox");
      	return false;
      }
      else
   	{
			random = Rand(2);
			switch(random)
			{
				case 0:
      			ReplaceWith(Other,"FoodFight.HalapenjoCan");
				break;
				case 1:
					ReplaceWith(Other,"FoodFight.Brocconade");
				break;
			}
      	return false;
      }
   }
   if(Other.Class==Class'SniperRifle')
   {
      ReplaceWith(Other,"FoodFight.HAnKARifle");
      return false;
   }
   if(Other.Class==Class'UT_BioRifle')
   {
      ReplaceWith(Other,"FoodFight.Saucier");
      return false;
   }
   if(Other.Class==Class'UT_Eightball')
   {
      ReplaceWith(Other,"FoodFight.Toastanator");
      return false;
   }
   if(Other.Class==Class'UT_FlakCannon')
   {
      ReplaceWith(Other,"FoodFight.SaladThrower");
      return false;
   }
   if(Other.Class==Class'WarHeadLauncher')
   {
   	if(UseLunchBox)
   	{
   	   ReplaceWith(Other,"FoodFight.lunchbox");
      	return false;
      }
      else
   	{
			random = Rand(2);
			switch(random)
			{
				case 0:
      			ReplaceWith(Other,"FoodFight.HalapenjoCan");
				break;
				case 1:
					ReplaceWith(Other,"FoodFight.Brocconade");
				break;
			}
      	return false;
      }
   }
   if(Other.Class==Class'Translocator')
   {
      return false;
   }
//Ammo pickups
	if(Other.Class==Class'BulletBox')
   {
   	if (FRand() < 0.25)
   	{
			ReplaceWith(Other,"FoodFight.KidneyBeanCan");
			return false;
		}
		else
		{
			ReplaceWith(Other,"FoodFight.PlumCan");
			return false;
		}
	}
	if(Other.Class==Class'Clip')
   {
   	if (FRand() < 0.25)
   	{
			ReplaceWith(Other,"FoodFight.KidneyBeanCan");
			return false;
		}
		else
		{
			ReplaceWith(Other,"FoodFight.PlumCan");
			return false;
		}
	}
   if(Other.Class==Class'RifleShell')
   {
   	if (FRand() < 0.25)
   	{
			ReplaceWith(Other,"FoodFight.KidneyBeanCan");
			return false;
		}
		else
		{
			ReplaceWith(Other,"FoodFight.PlumCan");
			return false;
		}
	}
   if(Other.Class==Class'RifleRound')
   {
   	if (FRand() < 0.25)
   	{
			ReplaceWith(Other,"FoodFight.KidneyBeanCan");
			return false;
		}
		else
		{
			ReplaceWith(Other,"FoodFight.PlumCan");
			return false;
		}
	}
   if(Other.Class==Class'BioAmmo')
   {
      ReplaceWith(Other,"FoodFight.SaucierAmmo");
      return false;
   }
   if(Other.Class==Class'BladeHopper')
   {
      ReplaceWith(Other,"FoodFight.CarrotPack");
      return false;
   }
   if(Other.Class==Class'FlakAmmo')
   {
      ReplaceWith(Other,"FoodFight.SaladPack");
      return false;
   }
   if(Other.Class==Class'MiniAmmo')
   {
      ReplaceWith(Other,"FoodFight.PeanutCan");
      return false;
   }
   if(Other.Class==Class'PAmmo')
   {
      ReplaceWith(Other,"FoodFight.SweetCherryCan");
      return false;
   }
	if(Other.Class==Class'RocketPack')
   {
      ReplaceWith(Other,"FoodFight.BreadPack");
      return false;
   }
   if(Other.Class==Class'ShockCore')
   {
      ReplaceWith(Other,"FoodFight.PapriNadeCylinder");
      return false;
   }
   if(Other.Class==Class'SuperShockCore')
   {
		random = Rand(2);
		switch(random)
		{
			case 0:
      		ReplaceWith(Other,"FoodFight.HalapenjoCan");
			break;
			case 1:
				ReplaceWith(Other,"FoodFight.Brocconade");
			break;
		}
      return false;
   }
	if(Other.Class==Class'WarheadAmmo')
   {
      ReplaceWith(Other,"FoodFight.HalapenjoCan");
      return false;
   }
//Health pickups
	if(Other.Class==Class'MedBox')
   {
      ReplaceWith(Other,"FoodFight.MilkCarton");
      return false;
   }
   if(Other.Class==Class'HealthPack')
   {
      ReplaceWith(Other,"FoodFight.JuiceBottleApple");
      return false;
   }
   if(Other.Class==Class'Armor2')
   {
      ReplaceWith(Other,"FoodFight.FFArmor");
      return false;
   }
   if(Other.Class==Class'ThighPads')
   {
      ReplaceWith(Other,"FoodFight.Apron");
      return false;
   }
   if(Other.Class==Class'UT_ShieldBelt')
   {
      ReplaceWith(Other,"FoodFight.Antacid");
      return false;
   }
   if(Other.Class==Class'Healthvial')
   {
   	randomA = Rand(2);
		switch(randomA)
		{
			case 0:
				ReplaceWith(Other,"FoodFight.JuiceBottleOrange");
				return false;
			break;
			case 1:
				ReplaceWith(Other,"FoodFight.JuiceBottleGrape");
				return false;
			break;
		}
      return false;
   }
//Assorted pickups
   if(Other.Class==Class'UDamage')
   {
      ReplaceWith(Other,"FoodFight.CoffeePickup");
      return false;
   }
   if((Other.Class==Class'UT_invisibility') || (Other.Class==Class'Invisibility') || (Other.Class==Class'UT_Stealth'))
   {
      ReplaceWith(Other,"FoodFight.PepperMill");
      return false;
   }
   if(Other.Class==Class'UT_JumpBoots')
   {
      ReplaceWith(Other,"FoodFight.JellyAmmo");
      return false;
   }
   bSuperRelevant = 0;
   return true;
}

//A jelly jumper is given to all players because of the way I coded it. When you see a jumper and pick it up
//you are actually getting the ammo class and not the jumper main class. Since you don't start with any ammo 
//for it you don't know it is in your inventory. Once you 'pick up' an ammo class you see the jumper and you 
//can toss it. When you do the ammo goes to zero and the jumper is no longer seen in your weapons. It's hacky, 
//I know but it works great.
function ModifyPlayer(Pawn Other)
{
   local inventory inv;

   GiveWeapon(Other,class'FoodFight.CanOpener');
   if (!Other.IsA('Bot'))
   {
   	GiveWeapon(Other,class'FoodFight.FryingPan');
   	GiveWeapon(Other,class'FoodFight.JellyJumper');
   }
	//loaded!
   if(GiveAllWeapons)
   {
      GiveWeapon(Other,class'FoodFight.PeanutGun');
      GiveWeapon(Other,class'FoodFight.PapriCannonGun');
      GiveWeapon(Other,class'FoodFight.HAnKARifle');
      GiveWeapon(Other,class'FoodFight.Toastanator');
      GiveWeapon(Other,class'FoodFight.SaladThrower');
      GiveWeapon(Other,class'FoodFight.Saucier');
      GiveWeapon(Other,class'FoodFight.Carroteer');
      //GiveWeapon(Other,class'');
   }

   if (NextMutator != None)
      NextMutator.ModifyPlayer(Other);
}

//Handler for the weapon giving
function GiveWeapon(Pawn PlayerPawn, class<TournamentWeapon> WeaponClass)
{
   local Weapon NewWeapon;

   if(PlayerPawn.FindInventoryType(WeaponClass) != None)
      return;
   newWeapon = Spawn(WeaponClass);
   if(newWeapon != None)
   {
      newWeapon.RespawnTime = 0.0;
      newWeapon.bHeldItem = true;
      newWeapon.GiveTo(PlayerPawn);
      newWeapon.SetSwitchPriority(PlayerPawn);
      newWeapon.WeaponSet(PlayerPawn);
      //don't add ammo to weapons that don't need them
      if (!newWeapon.IsA('FryingPan') || !newWeapon.IsA('JellyJumper') || !newWeapon.IsA('CanOpener'))
      {
	 		newWeapon.GiveAmmo(PlayerPawn);
			if (NewWeapon.Ammotype != None)
	 			newWeapon.AmmoType.AmmoAmount = newWeapon.AmmoType.MaxAmmo;
      }
      //Jelly is handled differently, see the weapon source code. We need to ensure it starts with zero ammo
      if (newWeapon.IsA('JellyJumper'))
	 		newWeapon.AmmoType.AmmoAmount = 0;
   }
}

defaultproperties
{
	UseLunchBox=True
   DMGameName="FoodFight!"
   DMLeftMessage=" has left the kitchen!!!"
   DMEnteredMessage=" has entered the kitchen!!!"
   GiveAllWeapons=False
}
