//
// The jelly jumper works in reverse. You spawn with an empty weapon and the pickup on the maps
// is actually the ammo class. This allows for proper respawn and replication, and lets the
// player pick up another after using the first one. Hacky I know but it works well.
class JellyJumper extends FFWeapon;

#exec MESH IMPORT MESH=JellyJumpPlatformPickup ANIVFILE=MODELS\JellyJumpPlatformPickup_a.3d DATAFILE=MODELS\JellyJumpPlatformPickup_d.3d X=0 Y=0 Z=0
#exec MESH ORIGIN MESH=JellyJumpPlatformPickup X=0 Y=0 Z=0

#exec MESH LODPARAMS MESH=JellyJumpPlatformPickup STRENGTH=0
#exec MESH SEQUENCE MESH=JellyJumpPlatformPickup SEQ=All 	STARTFRAME=0 NUMFRAMES=100
#exec MESH SEQUENCE MESH=JellyJumpPlatformPickup SEQ=Still 	STARTFRAME=1 NUMFRAMES=1
#exec MESH SEQUENCE MESH=JellyJumpPlatformPickup SEQ=Shoot 	STARTFRAME=1 NUMFRAMES=1
#exec MESH SEQUENCE MESH=JellyJumpPlatformPickup SEQ=Select STARTFRAME=1 NUMFRAMES=1
#exec MESH SEQUENCE MESH=JellyJumpPlatformPickup SEQ=Down 	STARTFRAME=1 NUMFRAMES=1
#exec MESH SEQUENCE MESH=JellyJumpPlatformPickup SEQ=Bounce STARTFRAME=2 NUMFRAMES=14

#exec MESHMAP NEW MESHMAP=JellyJumpPlatformPickup MESH=JellyJumpPlatformPickup
#exec MESHMAP SCALE MESHMAP=JellyJumpPlatformPickup X=0.025 Y=0.025 Z=0.05

#exec TEXTURE IMPORT NAME=jelly FILE=Textures/jelly.bmp GROUP=Skins FLAGS=2
#exec MESHMAP SETTEXTURE MESHMAP=JellyJumpPlatformPickup NUM=1 TEXTURE=jelly
#exec MESHMAP SETTEXTURE MESHMAP=JellyJumpPlatformPickup NUM=2 TEXTURE=jelly

//HUD Tile
#exec TEXTURE IMPORT NAME=JellyIcon FILE=TEXTURES\JellyHUD.bmp GROUP="Icons" MIPS=OFF

//Audio
#exec AUDIO IMPORT FILE="Sounds\JellyA.WAV" 		NAME=JellyA
#exec AUDIO IMPORT FILE="Sounds\JellyB.WAV" 		NAME=JellyB

var PapriNade currentShot;

function float RateSelf( out int bUseAltMode )
{
   if ((AmmoType != None) && (AmmoType.AmmoAmount <=0))
      return -2;
   bUseAltMode = 0;
   return (AIRating + FRand() * 0.05);
}

function Fire(float Value)
{
	local Paprinade newNade;

	if (AmmoType == None)
		GiveAmmo(Pawn(Owner));

	if (AmmoType.UseAmmo(1))
	{
		Gotostate('NormalFire');
		bCanClientFire = true;
		ClientFire(Value);
		newNade = PapriNade(ProjectileFire(ProjectileClass, ProjectileSpeed, bWarnTarget));
		if (newNade != None)
		{
			if (currentShot != None && !currentShot.bDeleteMe)
				newNade.nextNade = currentShot;
			currentShot = newNade;
		}
	}
}

simulated function PlayFiring()
{
   PlayOwnedSound(FireSound, SLOT_Misc, 1.7*Pawn(Owner).SoundDampening);
   PlayAnim('Down',0.7, 0.05);  //duplicate anim?
}

function AltFire(float Value)
{
}

simulated function PlayAltFiring()
{
}

simulated function PlayReloading()
{
}

simulated function PostRender(canvas C)
{
	local PlayerPawn P;
   local float Scale;
   local font CanvasFont;

   Super.PostRender(C);
   P = PlayerPawn(Owner);
   if (P != None)
   {
   	bOwnsCrossHair = true;
		Scale = C.ClipX/640;
		
		C.SetPos(0.5 * C.ClipX - 128 * Scale, 0.5 * C.ClipY - 128 * Scale );
		C.Style = ERenderStyle.STY_Translucent;
		C.DrawIcon(Texture'PapriReticle', Scale);
	}
}

defaultproperties
{
	WeaponDescription="Jelly jump pad for placeable jump spot"
	AmmoName=Class'JellyAmmo'
	PickupAmmoCount=0
	bAltWarnTarget=True
	bRapidFire=False
	FiringSpeed=1.000000
	FireOffset=(X=15.000000,Y=-13.000000,Z=-7.000000)
   ProjectileClass=class'FoodFight.JellyJumperPadFlying'
	AIRating=0.600000
   RefireRate=0.50000
   AltRefireRate=1.00000
	FireSound=Sound'JellyA'
   AltFireSound=Sound'UnrealI.BioRifle.GelShot'
   CockingSound=Sound'UnrealI.BioRifle.GelLoad'
   SelectSound=Sound'PapriPickup'
   DeathMessage="%o was blasted by the red hot thing thrown by %k"
	AutoSwitchPriority=5
	InventoryGroup=1
	PickupMessage="You got the Jelly Jumper!"
	ItemName="JellyJumper"
	PlayerViewOffset=(X=5.000000,Y=-2.200000,Z=-5.000000)
	PlayerViewMesh=JellyJumpPlatformPickup
   BobDamping=0.972000
	PickupViewMesh=JellyJumpPlatformPickup
	ThirdPersonMesh=JellyJumpPlatformPickup
	StatusIcon=JellyIcon
   PickupSound=Sound'PapriPickup'
	Icon=JellyIcon
	Mesh=JellyJumpPlatformPickup
	bNoSmooth=False
	CollisionRadius=25.00
	CollisionHeight=15.00
	PlayerViewScale=0.4
	ThirdPersonScale=0.60
}