//
class NeutralFlag extends FlagMutatorFlag config(FoodFight);

#exec AUDIO IMPORT NAME=TickTock FILE=SOUNDS\TickTock.wav

var byte LastTeam;
var float TimePassed;
var float CheckTimePassed;

var config bool bAllowFarting;
var config int  bFatRate;

replication
{
	reliable if (Role == ROLE_Authority)
		LastTeam;
}

function PostBeginPlay()
{
	Super(CTFFlag).PostBeginPlay();
	BurgerCTFGame(Level.Game).MutatorFlagInit(Self, True, True);
}

simulated function Tick(float DeltaTime)
{
	local BlackSmoke b;
	local SmallSpark2 c;

	if ( Level.bDropDetail )
	{
		Disable('Tick');
		Return;
	}
	if (FRand()<0.05)
	{
		b = Spawn(class'BlackSmoke');
		b.RemoteRole = ROLE_None;
		c = Spawn(class'SmallSpark2');
		c.RemoteRole = ROLE_None;
	}
}

function SendHome()
{
	Super.SendHome();
	LastTeam = Team;
}

state Dropped
{
	function Timer()
	{
		local pawn TeamMate;
	
		for (TeamMate = Level.PawnList; TeamMate != None; TeamMate = TeamMate.NextPawn)
			if (TeamMate.IsA('PlayerPawn'))
				PlayerPawn(TeamMate).ClientPlaySound(Sound'Announcer.BurgerReset');
		SendHome();
		BurgerCTFGame(Level.Game).MutatorFlagAutoReturn(Self);
		BroadcastLocalizedMessage(class'BurgerMessage', 1, None, None, None);
		if (Level.Game.WorldLog != None)
			Level.Game.WorldLog.LogSpecialEvent("flag_returned_timeout", 3);
		if (Level.Game.LocalLog != None)
			Level.Game.LocalLog.LogSpecialEvent("flag_returned_timeout", 3);
	}

	function Touch(Actor Other)
	{
		local CTFFlag aFlag;
		local Pawn aPawn;
		local NavigationPoint N;
		local int num, i;

		aPawn = Pawn(Other);
		if (aPawn != None && aPawn.bIsPlayer && aPawn.Health > 0 && !aPawn.IsInState('FeigningDeath'))
		{
			ClearAssistance(aPawn.PlayerReplicationInfo.Team);
			aPawn.MoveTimer = -1;
			Holder = aPawn;
			Holder.PlayerReplicationInfo.HasFlag = self;
			LastTeam = Holder.PlayerReplicationInfo.Team;
			LastHolder = Holder;
			SetHolderLighting();
			if (Holder.IsA('Bot'))
			{
				Bot(Holder).AlternatePath = None;
				Holder.SendTeamMessage(None, 'OTHER', 2, 10);
			}
			else if (Holder.IsA('TournamentPlayer') && TournamentPlayer(Holder).bAutoTaunt)
				Holder.SendTeamMessage(None, 'OTHER', 2, 10);
			
			BroadcastLocalizedMessage(class'BurgerMessage', 3, Holder.PlayerReplicationInfo, None, BurgerCTFGame(Level.Game).Teams[Holder.PlayerReplicationInfo.Team]);
			if (Level.Game.WorldLog != None)
				Level.Game.WorldLog.LogSpecialEvent("flag_pickedup", Holder.PlayerReplicationInfo.PlayerID, CTFGame(Level.Game).Teams[Holder.PlayerReplicationInfo.Team].TeamIndex);
			if (Level.Game.LocalLog != None)
				Level.Game.LocalLog.LogSpecialEvent("flag_pickedup", Holder.PlayerReplicationInfo.PlayerID, CTFGame(Level.Game).Teams[Holder.PlayerReplicationInfo.Team].TeamIndex);
			GotoState('Held');
			BurgerCTFGame(Level.Game).MutatorFlagPickup(Self, Holder);
		}
	}
}

auto state Home
{
	function Timer()
	{
		if (VSize(Location - HomeBase.Location) > 10)
			SendHome();
	}

	function Touch(Actor Other)
	{
		local CTFFlag aFlag;
		local Pawn aPawn;
		local NavigationPoint N;
		local int i;
		local float totalweight, selection, PartialWeight;
		local Bot B;

		aPawn = Pawn(Other);
		if (aPawn != None && aPawn.bIsPlayer && aPawn.Health > 0)
		{
			Holder = aPawn;
			Holder.MoveTimer = -1;
			Holder.PlayerReplicationInfo.HasFlag = self;
			LastTeam = Holder.PlayerReplicationInfo.Team;
			LastHolder = Holder;
			Holder.MakeNoise(2.0);
			SetHolderLighting();
			BurgerCTFGame(Level.Game).MutatorFlagSteal(Self, Holder);
			B = Bot(Holder);
			if (B != None)
			{
				B.AlternatePath = None;
				if (FRand() < 0.5)
				{
					for (N = Level.NavigationPointList; N != None; N = N.nextNavigationPoint)
						if (N.IsA('AlternatePath') && (AlternatePath(N).bTwoWay || AlternatePath(N).team == Holder.PlayerReplicationInfo.team))
						{
							if (AlternatePath(N).bTwoWay && AlternatePath(N).Team != Holder.PlayerReplicationInfo.Team)
								TotalWeight += 4000;
							TotalWeight += AlternatePath(N).SelectionWeight;
						}
						selection = TotalWeight * FRand();
						for (N=Level.NavigationPointList; N!=None; N=N.nextNavigationPoint)
							if (N.IsA('AlternatePath') && (AlternatePath(N).bTwoWay || AlternatePath(N).Team == Holder.PlayerReplicationInfo.Team))
							{
								B.AlternatePath = AlternatePath(N);
								if (AlternatePath(N).bTwoWay && (AlternatePath(N).team != Holder.PlayerReplicationInfo.team))
									PartialWeight += 4000;
								PartialWeight += AlternatePath(N).SelectionWeight;
								if (PartialWeight > selection)
									break;
							}
				}
				B.SendTeamMessage(None, 'OTHER', 2, 10);
			}
			else if (Holder.IsA('TournamentPlayer') && TournamentPlayer(Holder).bAutoTaunt)
				Holder.SendTeamMessage(None, 'OTHER', 2, 10);
			if (Level.Game.WorldLog != None)
				Level.Game.WorldLog.LogSpecialEvent("flag_taken", Holder.PlayerReplicationInfo.PlayerID, BurgerCTFGame(Level.Game).Teams[Holder.PlayerReplicationInfo.Team].TeamIndex);
			if (Level.Game.LocalLog != None)
				Level.Game.LocalLog.LogSpecialEvent("flag_taken", Holder.PlayerReplicationInfo.PlayerID, BurgerCTFGame(Level.Game).Teams[Holder.PlayerReplicationInfo.Team].TeamIndex);
			BroadcastLocalizedMessage(class'BurgerMessage', 3, Holder.PlayerReplicationInfo, None, BurgerCTFGame(Level.Game).Teams[Holder.PlayerReplicationInfo.Team]);
			GotoState('Held');
		}
	}
}

state Held
{
	function BeginState()
	{
		Super.BeginState();
		BurgerCTFGame(Level.Game).MutatorFlagBeginCarry(Self, Holder);
	}
	
	function Tick(float DeltaTime)
	{
		local float decision;
		local EffectParticleBase AssGas;

		Super.Tick(DeltaTime);
		if ( Holder != None ) 
		{
			SetLocation(Holder.Location);
			Velocity = Holder.Velocity;
		}

		if(bAllowFarting)
		{
			TimePassed += DeltaTime;
			CheckTimePassed += DeltaTime;

			if (TimePassed >= 0.3)
			{
		 		if (Holder.Fatness >= 240)
		 		{
		 			Holder.Fatness = 255;
		 			if (FRand()<0.1)
		 			{
		 				decision = FRand();
		 				if (decision < 0.5)
		 				{
		 					Holder.PlaySound(Sound'amb2g');
		 					AssGas = spawn(class'EffectParticleBase', Holder, , Holder.Location, Holder.Rotation);
		 				}
		 				else
		 				{
		 					Holder.PlaySound(Sound'nearby1g');
		 					AssGas = spawn(class'EffectParticleBase', Holder, , Holder.Location, Holder.Rotation);
		 				}
					}

		 		}
		 		else
		 			Holder.Fatness += bFatRate;
		 		TimePassed = 0;
			}
		}
	}

	function EndState()
	{
		BurgerCTFGame(Level.Game).MutatorFlagEndCarry(Self, LastHolder);
		Super.EndState();
	}
}

defaultproperties
{
   FlagName="Neutral Flag"
   FlagMessage=Class'BurgerMessage'
   Team=3
   AmbientGlow=128
   AmbientSound=TickTock
   SoundRadius=30
   SoundVolume=255
   LightBrightness=100
   LightSaturation=255
   Mesh=LodMesh'Burger'
   DrawScale=0.400000
   bAllowFarting=True
   bFatRate=5
}
