//
//large alt fire glob
class RedGlobBig extends RedGlob;

var int NumSplash;
var vector SpawnPoint;
var() bool CanHitInstigator;
var() float HitInstigatorTimeOut;

auto state Flying
{
   function ProcessTouch (Actor Other, vector HitLocation)
   {
	 	local ObjectsFlames F;
		local bool bThere;

		//this check is used to add a short timer when the glob is shot so that it will not touch anything. Why is this 
		//important? Well it prevents someone moving very fast from colliding with their own shot if they shoot in the 
		//direction they are running. By setting "CanHitInstigator" to false in the def props I can totally prevent
		//self damage. This could be handy with a grapple type weapon for instance
		if ((!CanHitInstigator || (CanHitInstigator && (Default.LifeSpan - LifeSpan) < HitInstigatorTimeOut)) && Other == Instigator)
		return;

		if (!Region.Zone.bWaterZone)
		{
			//only spawns fire on these classes
   	   if (Other.IsA('Pawn') || (Other.IsA('Decoration') && (!Other.IsA('CTFFlag'))) || (Other.IsA('Carcass')))
			{
				if (Other.DrawScale < 1.25 && Other.DrawScale > 0.75)
				{
					ForEach RadiusActors(Class'ObjectsFlames', F, 8.0, Other.Location)
					{
						if (F.Owner == Other)
						{
							//simple check to keep an actor from getting multiple flames on them
							bThere=True;
							break;
						}
					}
					//not currently on fire so let's add some
					if (!bThere)
						F = Spawn(Class'ObjectsFlames', Other,, Other.Location);
				}
			}
		}
      if (Other.IsA('RedGlobSplash'))
         return;
      if (Pawn(Other)!=Instigator || bOnGround)
         Global.Timer();
   }
   simulated function HitWall(vector HitNormal, actor Wall)
	{
      SetPhysics(PHYS_None);
      MakeNoise(1);
      bOnGround = True;
      PlaySound(ImpactSound);
      SetWall(HitNormal, Wall);
      if ( DrawScale > 1 )
         NumSplash = int(2 * DrawScale) - 1;
      SpawnPoint = Location + 5 * HitNormal;
      DrawScale= FMin(DrawScale, 3.0);
      if ( NumSplash > 0 )
      {
         SpawnSplash();
         if ( NumSplash > 0 )
         	SpawnSplash();
      }
      GoToState('OnSurface');
	}
}

//some effects
function SpawnSplash()
{
   local vector Start;

   NumSplash--;
   Start = SpawnPoint + 4 * VRand();
   Spawn(class'RedGlobSplash',,,Start,Rotator(Start - Location));
}

state OnSurface
{
   function Tick(float DeltaTime)
   {
      if (NumSplash > 0)
      {
         SpawnSplash();
         if (NumSplash > 0)
            SpawnSplash();
         else
            Disable('Tick');
      }
      else
         Disable('Tick');
   }

	//glob has landed/hit on something and is now touching it
	function ProcessTouch (Actor Other, vector HitLocation)
   {
		local ObjectsFlames F;
		local bool bThere;
		
		if (!Region.Zone.bWaterZone)
		{
   	   if (Other.IsA('Pawn') || (Other.IsA('Decoration') && (!Other.IsA('CTFFlag'))) || (Other.IsA('Carcass')))
			{
				if (Other.DrawScale < 1.25 && Other.DrawScale > 0.75)
				{
					ForEach RadiusActors(Class'ObjectsFlames', F, 8.0, Other.Location)
					{
						if (F.Owner == Other)
						{
							bThere=True;
							break;
						}
					}
					if (!bThere) 
						F = Spawn(Class'ObjectsFlames', Other,, Other.Location);
				}
			}
		}
      if ( Other.IsA('RedGlobSplash') )
         return;
      GotoState('Exploding');
   }
}

defaultproperties
{
	HitInstigatorTimeOut=0.500000
	CanHitInstigator=True
   speed=840.000000
   MaxSpeed=1500.000000
   Damage=+0025.000000
   MomentumTransfer=30000
}
